@@ -12,7 +12,7 @@ const char *config_defaultdomain()
 {
 	if (!defaultdomain)
 	{
-	char	*f=config_search("defaultdomain");
+	char	*f=config_localfilename("defaultdomain");
 
 		if ((defaultdomain=config_read1l(f)) == 0)
 			defaultdomain=config_me();
