@@ -12,7 +12,7 @@ const char *config_msgidhost()
 {
 	if (!msgidhost)
 	{
-	char	*f=config_search("msgidhost");
+	char	*f=config_localfilename("msgidhost");
 
 		if ((msgidhost=config_read1l(f)) == 0)
 			msgidhost=config_me();
