@@ -59,6 +59,7 @@ int ctlfile_openit(ino_t, time_t, struct ctlfile *, int);
 int ctlfile_openfn(const char *, struct ctlfile *, int, int);
 void ctlfile_close(struct ctlfile *);
 void ctlfile_append(struct ctlfile *, const char *);
+int ctlfile_setvhost(struct ctlfile *);
 
 void ctlfile_append_reply(struct ctlfile *, unsigned,
 	const char *, char, const char *);
@@ -105,6 +106,7 @@ time_t ctlfile_getnextattempt(struct ctlfile *);
 #define	COMCTLFILE_DELDEFERRED	'D'
 #define	COMCTLFILE_DELCOMPLETE	'C'
 #define	COMCTLFILE_VERP		'V'
+#define COMCTLFILE_VHOST	'v'
 #define	COMCTLFILE_CANCEL_MSG	'X'
 #define	COMCTLFILE_CANCELLED_ACK 'x'
 #define	COMCTLFILE_NEXTATTEMPT	'A'
