@@ -4,7 +4,7 @@
 */
 
 #if	HAVE_CONFIG_H
-#include	"config.h"
+#include	"courier_lib_config.h"
 #endif
 #include	"courier.h"
 #include	"localstatedir.h"
@@ -52,7 +52,7 @@ static char buf[MAXLONGSIZE + 10 + sizeof(MSGQDIR "/") ];
 
 const char *qmsgqname(ino_t i, time_t t)
 {
-static char buf[MAXLONGSIZE*3 + 25 ];
+static char buf[MAXLONGSIZE*3 + 25 + sizeof(MSGQDIR "/") ];
 
 	strcat(strcpy(buf, qmsgqdir(t)), "/C");
 	strcat(buf, strinode(i));
