@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2010 Double Precision, Inc.
+** Copyright 1998 - 2012 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -69,6 +69,13 @@ static time_t teergrube=INIT_TEERGRUBE;
 
 extern const char *externalauth();
 
+static const char *truncate_ipv6(const char *tcp)
+{
+	if (strncmp(tcp, "::ffff:", 7) == 0)
+		tcp += 7;
+	return tcp;
+}
+
 static const char *smtp_externalauth()
 {
 	const char *p;
@@ -1048,13 +1055,19 @@ int	authenticated=0;
 
 	else
 	{
-	const char *p;
+		const char *p=getenv("TCPLOCALIP");
+
+		if (p)
+			p=truncate_ipv6(p);
+
+		if (p && *p && config_has_vhost(p))
+			config_set_local_vhost(p);
 
 		clog_open_syslog("courieresmtpd");
 
 		if ((p=getenv("ESMTP_TLS")) && atoi(p))
 		{
-			const char *p=getenv("TLS_CONNECTED_PROTOCOL");
+			p=getenv("TLS_CONNECTED_PROTOCOL");
 
 			tls=1;
 			/* This is the smtps service, and we don't need to
@@ -1172,11 +1185,20 @@ int	authenticated=0;
 		char	buf1[NUMBUFSIZE+40];
 		char	buf2[NUMBUFSIZE];
 
-			if (!starttls)
+		        if (!starttls ||
+			    ((p=getenv("TLS_DEBUG_FAIL_STARTTLS_HARD")) &&
+			     *p == '1'))
 			{
 				addiovec_error("540 TLS not available.");
 				continue;
 			}
+
+		        if ((p=getenv("TLS_DEBUG_FAIL_STARTTLS_SOFT")) &&
+			    *p == '1')
+			{
+				addiovec_error("440 TLS not available.");
+				continue;
+			}
 			if (libmail_streampipe(pipefd))
 			{
 				addiovec_error("432 libmail_streampipe() failed.");
