@@ -181,7 +181,7 @@ char	*p;
 
 	if (!percenthack)	/* First time */
 	{
-		p=config_search("esmtppercentrelay");
+		p=config_localfilename("esmtppercentrelay");
 
 		percenthack=readfile(p, 0);
 		free(p);
@@ -191,7 +191,7 @@ char	*p;
 		else
 			percenthack="";
 		dbobj_init(&percenthackdat);
-		p=config_search("esmtppercentrelay.dat");
+		p=config_localfilename("esmtppercentrelay.dat");
 		dbobj_open(&percenthackdat, p, "R");
 		free(p);
 	}
@@ -390,7 +390,7 @@ static struct dbobj acceptdomainsdb;
 
 	if (!acceptdomains)
 	{
-        char *filename=config_search("esmtpacceptmailfor");
+        char *filename=config_localfilename("esmtpacceptmailfor");
 	char	*buf;
 
 		buf=readfile(filename, 0);
@@ -405,7 +405,7 @@ static struct dbobj acceptdomainsdb;
                 }
 
 		dbobj_init(&acceptdomainsdb);
-		filename=config_search("esmtpacceptmailfor.dat");
+		filename=config_localfilename("esmtpacceptmailfor.dat");
 		dbobj_open(&acceptdomainsdb, filename, "R");
 		free(filename);
 	}
