@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2010 Double Precision, Inc.
+** Copyright 1998 - 2012 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -131,6 +131,17 @@ void rfc2045_error(const char *p)
 	_exit(1);
 }
 
+static void setconfig()
+{
+       esmtpkeepaliveping=config_time_esmtpkeepaliveping();
+       cmd_timeout=config_time_esmtptimeout();
+       connect_timeout=config_time_esmtpconnect();
+       cmd_timeout=config_time_esmtptimeout();
+       helo_timeout=config_time_esmtphelo();
+       data_timeout=config_time_esmtpdata();
+       quit_timeout=config_time_esmtpquit();
+}
+
 /* We get here as a new child process of the courieresmtp driver */
 
 void esmtpchild(unsigned childnum)
@@ -148,13 +159,7 @@ unsigned long mypid=(unsigned long)getpid();
 		clog_msg_errno();
 	sockfd= -1;
 
-	esmtpkeepaliveping=config_time_esmtpkeepaliveping();
-	cmd_timeout=config_time_esmtptimeout();
-	connect_timeout=config_time_esmtpconnect();
-	cmd_timeout=config_time_esmtptimeout();
-	helo_timeout=config_time_esmtphelo();
-	data_timeout=config_time_esmtpdata();
-	quit_timeout=config_time_esmtpquit();
+	setconfig();
 
 	net_timeout=0;
 
@@ -193,6 +198,14 @@ unsigned long mypid=(unsigned long)getpid();
 
 		if (ctlfile_openi(del->inum, &ctf, 0) == 0)
 		{
+			int changed_vhosts=ctlfile_setvhost(&ctf);
+
+			if (changed_vhosts)
+			{
+				quit();
+				setconfig();
+			}
+
 			sendesmtp(del, &ctf);
 			ctlfile_close(&ctf);
 		}
@@ -292,6 +305,8 @@ static int get_sourceaddr(int af,
 {
 	int rc;
 	RFC1035_ADDR in;
+	const char *vhost=config_get_local_vhost();
+	char vhost_ip_buf[100];
 	const char *buf=getenv(
 #if RFC1035_IPV6
 
@@ -300,6 +315,41 @@ static int get_sourceaddr(int af,
 #endif
 			       "SOURCE_ADDRESS");
 
+	char *source=config_localfilename(
+#if RFC1035_IPV6
+
+					  !IN6_IS_ADDR_V4MAPPED(dest_addr)
+					  ? "ip6out":
+#endif
+					  "ipout");
+	FILE *fp;
+	fp=fopen(source, "r");
+	free(source);
+
+	if (fp)
+	{
+		char *p;
+
+		/*
+		** If ip[6]out.xxxx exists, read the IP address from it.
+		*/
+		vhost_ip_buf[0]=0;
+		if (fgets(vhost_ip_buf, sizeof(vhost_ip_buf), fp) == NULL)
+			vhost_ip_buf[0]=0;
+		fclose(fp);
+
+		p=strchr(vhost_ip_buf, '\n');
+		if (p) *p=0;
+
+		/*
+		** If the file is empty, use the vhost IP address itself.
+		*/
+		if (vhost_ip_buf[0])
+			buf=vhost_ip_buf;
+		else if (vhost && *vhost)
+			buf=vhost;
+	}
+
 	if (buf && strcmp(buf, "0")) {
 		rc = rfc1035_aton(buf, &in);
 		if (rc != 0)
@@ -429,6 +479,8 @@ static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 	{
 		struct rfc1035_mxlist *mxlist, *p, *q;
 		int static_route= smtproute != NULL;
+		struct rfc1035_res res;
+		int rc;
 
 		errno=0;	/* Detect network failures */
 
@@ -440,9 +492,16 @@ static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 		if (!auth_key)
 			clog_msg_errno();
 
-		rfc1035_init_resolv(&rfc1035_default_resolver);
-		switch (rfc1035_mxlist_create(&rfc1035_default_resolver,
-			auth_key, &mxlist))	{
+		rfc1035_init_resolv(&res);
+
+		rc=rfc1035_mxlist_create_x(&res,
+					   auth_key,
+					   RFC1035_MX_AFALLBACK |
+					   RFC1035_MX_IGNORESOFTERR,
+					   &mxlist);
+		rfc1035_destroy_resolv(&res);
+
+		switch (rc)	{
 		case RFC1035_MX_OK:
 			break;
 		case RFC1035_MX_HARDERR:
@@ -708,19 +767,22 @@ static void connection_closed(struct moduledel *del, struct ctlfile *ctf)
 
 /* Record an SMTP error for all the recipients */
 
-static void smtp_error1(struct moduledel *del, struct ctlfile *ctf,
-	const char *msg, int n)
-{
-	if (*msg == '5')
-		hard_error1(del, ctf, msg, n);
-	else
-		soft_error1(del, ctf, msg, n);
-}
-
 static void smtp_error(struct moduledel *del, struct ctlfile *ctf,
-	const char *msg)
+		       const char *msg, int errcode)
 {
-	smtp_error1(del, ctf, msg, -1);
+	if (!msg)
+	{
+		connect_error(del, ctf);
+		return;
+	}
+
+	if (errcode == 0)
+		errcode= *msg;
+
+	if (errcode == '5')
+		hard_error1(del, ctf, msg, -1);
+	else
+		soft_error1(del, ctf, msg, -1);
 }
 
 
@@ -782,7 +844,9 @@ static void connect_error1(struct moduledel *del, struct ctlfile *ctf, int n)
 #if	HAVE_STRERROR
 
 	if (errno)
-		soft_error1(del, ctf, strerror(errno), n);
+		soft_error1(del, ctf, errno == ECONNRESET
+			    ? "Network connection shut down by the remote mail server"
+			    : strerror(errno), n);
 	else
 #endif
 		soft_error1(del, ctf, "Connection closed by remote host.", n);
@@ -934,6 +998,7 @@ unsigned cnt, i;
 			{
 				sox_close(sockfd);
 				sockfd= -1;
+				errno=ECONNRESET;
 				return (0);
 			}
 			p = --mybuf_ptr( &sockbuf );
@@ -1166,8 +1231,17 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 
 	if (strcmp(p, "*") == 0)
 	{
+		struct rfc1035_res res;
+		int rc;
+
+		rfc1035_init_resolv(&res);
+
 		p=buf;
-		if (rfc1035_ptr(&rfc1035_default_resolver, &laddr, buf) != 0)
+		rc=rfc1035_ptr(&res, &laddr, buf);
+
+		rfc1035_destroy_resolv(&res);
+
+		if (rc != 0)
 		{
 			char *q;
 
@@ -1237,7 +1311,7 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 			if ((p=readline()) == 0)
 				return (1);
 		}
-		smtp_error(del, ctf, p);
+		smtp_error(del, ctf, p, 0);
 		quit();
 		return (-1);	/*
 				** Let the caller consider this a hard error,
@@ -1392,7 +1466,7 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 		sent(del, ctf, "SECURITY=STARTTLS REQUESTED FOR THIS MESSAGE");
 		smtp_msg(del, ctf);
 		smtp_error(del, ctf,
-			   "500 Unable to set minimum security level.");
+			   "500 Unable to set minimum security level.", 0);
 		quit();
 		return (-1);
 	}
@@ -1410,6 +1484,56 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 	return (0);
 }
 
+static void report_broken_starttls(const char *reply)
+{
+	const char *cmd=getenv("TLS_ERROR_REPORT");
+	const char *shell;
+
+	char	buf[RFC1035_NTOABUFSIZE];
+	pid_t p;
+
+	if (!cmd || !*cmd)
+		return;
+
+	p=fork();
+
+	if (p < 0)
+	{
+		clog_msg_start_err();
+		clog_msg_str("fork() failed");
+		clog_msg_send();
+		return;
+	}
+
+	if (p > 0)
+	{
+		wait(NULL); /* Better be this child */
+		return;
+	}
+
+	close(sockfd);
+
+	p=fork();
+
+	if (p)
+		_exit(0);
+
+	sockipname(buf);
+
+	setenv("ERROR_HOST", sockfdaddrname ? sockfdaddrname:"", 1);
+	setenv("ERROR_IP", buf, 1);
+	setenv("ERROR_CODE", "STARTTLS", 1);
+	setenv("ERROR_TEXT", reply, 1);
+
+	shell=getenv("SHELL");
+
+	if (!shell || !*shell)
+		shell="/bin/sh";
+
+	execl(shell, shell, "-c", cmd, NULL);
+	_exit(1);
+}
+
 static int starttls(struct moduledel *del, struct ctlfile *ctf,
 	const char *hostname)
 {
@@ -1431,6 +1555,11 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 	int restore_origcert=0;
 	int n;
 
+	const char *tls_error_handle=getenv("TLS_ERROR_HANDLE");
+
+	if (!tls_error_handle)
+		tls_error_handle="";
+
 	if (libmail_streampipe(pipefd))
 	{
 		perror("libmail_streampipe");
@@ -1444,6 +1573,9 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 		connect_error(del, ctf);
 		close(pipefd[0]);
 		close(pipefd[1]);
+
+		report_broken_starttls("(none)");
+
 		return (1);
 	}
 
@@ -1452,15 +1584,22 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 		talking(del, ctf);
 		sent(del, ctf, "STARTTLS");
 		smtp_msg(del, ctf);
+		close(pipefd[0]);
+		close(pipefd[1]);
+
+		report_broken_starttls(p);
+
 		while (!ISFINALLINE(p))
 		{
 			reply(del, ctf, p);
 			if ((p=readline()) == 0)
 				break;
 		}
-		smtp_error(del, ctf, p);
-		close(pipefd[0]);
-		close(pipefd[1]);
+
+		if (*tls_error_handle == 'i')
+			return 0;
+
+		smtp_error(del, ctf, p, *tls_error_handle == 's' ? '4':0);
 		return (-1);
 	}
 
@@ -1505,13 +1644,13 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 		if (!p || !*p)
 		{
 			static const char fail[]=
-				"Unable to set minimum security"
+				"500 Unable to set minimum security"
 				" level.\n";
 
 			talking(del, ctf);
 			sent(del, ctf, "STARTTLS");
 			smtp_msg(del, ctf);
-			smtp_error(del, ctf, fail);
+			smtp_error(del, ctf, fail, 0);
 			sox_close(sockfd);
 			sockfd= -1;
 			close(pipefd[0]);
@@ -1574,8 +1713,10 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 		sent(del, ctf, "STARTTLS");
 
 		smtp_msg(del, ctf);
-		strcat(strcpy(tmperrbuf, "500 "), cinfo.errmsg);
-		smtp_error(del, ctf, tmperrbuf);
+		strcat(strcpy(tmperrbuf,
+			      *tls_error_handle == 's' ? "400 ":"500 "),
+		       cinfo.errmsg);
+		smtp_error(del, ctf, tmperrbuf, 0);
 		sox_close(sockfd);
 		sockfd= -1;
 		couriertls_destroy(&cinfo);
@@ -1666,7 +1807,7 @@ unsigned line_num;
 				return (-1);
 			}
 		}
-		smtp_error(del, ctf, p);
+		smtp_error(del, ctf, p, 0);
 		return (-1);
 	}
 
@@ -2779,7 +2920,7 @@ int	rc;
 				return (-1);
 			}
 		}
-		smtp_error(del, ctf, p);
+		smtp_error(del, ctf, p, 0);
 		quit();
 		return (-1);
 	}
@@ -2824,7 +2965,7 @@ const char *p=readline();
 			return (0);
 		}
 	}
-	smtp_error(x->del, x->ctf, p);
+	smtp_error(x->del, x->ctf, p, 0);
 	return (0);
 }
 
