@@ -52,15 +52,12 @@
 #endif
 #endif
 
-static const char rcsid[]="$Id: faxconvert.c,v 1.7 2009/11/15 02:10:38 mrsam Exp $";
 
 #include "filterbindir.h"
 #include "faxtmpdir.h"
 
 #define SUBTMPDIR FAXTMPDIR "/.tmp"
 
-extern const struct unicode_info *default_charset;
-
 void rfc2045_error(const char *errmsg)
 {
 	fprintf(stderr, "faxconvert: %s\n", errmsg);
@@ -806,8 +803,7 @@ static int convert_cover_page(struct rfc2045 *rfcp,
 
 			hsubject=rfc822_display_hdrvalue_tobuf(hbuf.header,
 							       hbuf.value,
-							       default_charset
-							       ->chset,
+							       unicode_default_chset(),
 							       NULL, NULL);
 
 			if (!hsubject)
@@ -928,12 +924,12 @@ static char *convnames(const char *pp)
 
 	al=1;
 
-	if (rfc822_display_namelist(a, default_charset->chset,
+	if (rfc822_display_namelist(a, unicode_default_chset(),
 				    cnt_names, &al) == 0)
 	{
 		q=p=courier_malloc(al+1);
 
-		if (rfc822_display_namelist(a, default_charset->chset,
+		if (rfc822_display_namelist(a, unicode_default_chset(),
 					    save_names, &q) == 0)
 		{
 			if (q - p >= 2 && q[-1] == ' ' && q[-2] == ',')
