@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -67,8 +67,7 @@ char *cmd;
 		if (p)
 			exit(0);
 		close(pipefd[0]);
-		close(1);
-		dup(pipefd[1]);
+		dup2(pipefd[1], 1);
 		close(pipefd[1]);
 		if ((env=getenv("UFLINE")) != 0)
 			printf("%s\n", env);
@@ -84,8 +83,7 @@ char *cmd;
 	}
 
 	close(pipefd[1]);
-	close(0);
-	dup(pipefd[0]);
+	dup2(pipefd[0], 0);
 	close(pipefd[0]);
 
 	while (wait(&waitstat) != p)
