@@ -1,9 +1,8 @@
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: import.c,v 1.2 2001/03/21 22:44:18 mrsam Exp $";
 
 #include	"config.h"
 #include	<stdio.h>
@@ -20,15 +19,15 @@ static const char rcsid[]="$Id: import.c,v 1.2 2001/03/21 22:44:18 mrsam Exp $";
 #include	"unicode/unicode.h"
 #include	"numlib/numlib.h"
 
-extern int gpg_stdin, gpg_stdout, gpg_stderr;
-extern pid_t gpg_pid;
+extern int libmail_gpg_stdin, libmail_gpg_stdout, libmail_gpg_stderr;
+extern pid_t libmail_gpg_pid;
 
 
 /*
 ** Import a key.
 */
 
-int gpg_import_start(const char *gpgdir, int secret)
+int libmail_gpg_import_start(const char *gpgdir, int secret)
 {
 	char *argvec[5];
 
@@ -45,27 +44,27 @@ int gpg_import_start(const char *gpgdir, int secret)
 	}
 #endif
 
-	if (gpg_fork(&gpg_stdin, &gpg_stdout, NULL, gpgdir, argvec) < 0)
+	if (libmail_gpg_fork(&libmail_gpg_stdin, &libmail_gpg_stdout, NULL, gpgdir, argvec) < 0)
 		return (-1);
 	return (0);
 }
 
-int gpg_import_do(const char *p, size_t n,
+int libmail_gpg_import_do(const char *p, size_t n,
 		  int (*dump_func)(const char *, size_t, void *),
 		  void *voidarg)
 {
-	if (gpg_write(p, n, dump_func, dump_func, NULL, 0, voidarg))
+	if (libmail_gpg_write(p, n, dump_func, dump_func, NULL, 0, voidarg))
 	{
-		gpg_cleanup();
+		libmail_gpg_cleanup();
 		return (-1);
 	}
 	return (0);
 }
 
-int gpg_import_finish(int (*dump_func)(const char *, size_t, void *),
+int libmail_gpg_import_finish(int (*dump_func)(const char *, size_t, void *),
 		      void *voidarg)
 {
-	if (gpg_read(dump_func, dump_func, NULL, 0, voidarg))
+	if (libmail_gpg_read(dump_func, dump_func, NULL, 0, voidarg))
 		return (-1);
-	return (gpg_cleanup());
+	return (libmail_gpg_cleanup());
 }
