@@ -1,9 +1,8 @@
 /*
-** Copyright 2001-2003 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: list.c,v 1.4 2004/04/25 03:37:15 mrsam Exp $";
 
 #include	"config.h"
 #include	<stdio.h>
@@ -12,6 +11,7 @@ static const char rcsid[]="$Id: list.c,v 1.4 2004/04/25 03:37:15 mrsam Exp $";
 #include	<unistd.h>
 #include	<signal.h>
 #include	<ctype.h>
+#include	<time.h>
 #include	<errno.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
@@ -40,7 +40,7 @@ static void definit(struct gpg_list_info *arg)
 
 #define DEFINIT(c, a) if (!(c)) (c)=(a)
 
-	DEFINIT(arg->charset, &unicode_ISO8859_1);
+	DEFINIT(arg->charset, "iso-8859-1");
 	DEFINIT(arg->disabled_msg, "[ This key is disabled ]");
 	DEFINIT(arg->revoked_msg, "[ This key is revoked ]");
 	DEFINIT(arg->expired_msg, "[ This key is expired ]");
@@ -247,6 +247,7 @@ static int nyb(int c)
 }
 
 static int append_key(struct gpg_callbacklistinfo *, const char *);
+static int append_date(struct gpg_callbacklistinfo *, const char *);
 
 static int dolist_callback(char *line, void *vp1,
 			   struct gpg_list_info *vp)
@@ -257,11 +258,11 @@ static int dolist_callback(char *line, void *vp1,
 	char *trust;
 	char *length;
 	int algo;
-	char *keyid;
+	/*char *keyid;*/
 	char *crdate;
 	char *expdate;
-	char *localid;
-	char *ownertrust;
+	/*char *localid;*/
+	/*char *ownertrust;*/
 	char *userid;
 	char *p;
 	const char *stat;
@@ -298,11 +299,11 @@ static int dolist_callback(char *line, void *vp1,
 	trust=p; p=nextword(p);
 	length=p; p=nextword(p);
 	algo=atoi(p); p=nextword(p);
-	keyid=p; p=nextword(p);
+	/*keyid=p;*/ p=nextword(p);
 	crdate=p; p=nextword(p);
 	expdate=p; p=nextword(p);
-	localid=p; p=nextword(p);
-	ownertrust=p; p=nextword(p);
+	/*localid=p;*/ p=nextword(p);
+	/*ownertrust=p;*/ p=nextword(p);
 	userid=p; p=nextword(p);
 
 	{
@@ -388,12 +389,13 @@ static int dolist_callback(char *line, void *vp1,
 			return (-1);
 	}
 
-	userid=unicode_cfromutf8(vp->charset, userid, NULL);
+	userid=libmail_u_convert_fromutf8(userid, vp->charset, NULL);
 	if (!userid)
 		return (-1);
 
 	if (strcmp(rectype, "pub") == 0 ||
-	    strcmp(rectype, "sec") == 0)
+	    strcmp(rectype, "sec") == 0 ||
+	    (strcmp(rectype, "uid") == 0 && gci->shortname[0] == 0))
 	{
 		gci->shortname[0]=0;
 		strncat(gci->shortname, userid,
@@ -402,9 +404,9 @@ static int dolist_callback(char *line, void *vp1,
 	}
 
 	if (append_key(gci, " ")
-	    || append_key(gci, atoi(crdate) ?  crdate :"          ")
+	    || append_date(gci, crdate)
 	    || append_key(gci, " ")
-	    || append_key(gci, atoi(expdate) ? expdate:"          ")
+	    || append_date(gci, expdate)
 	    || append_key(gci, " ")
 	    || append_key(gci, userid)
 	    || append_key(gci, "\n"))
@@ -434,6 +436,41 @@ static char *nextword(char *p)
 	return (p);
 }
 
+static int append_date(struct gpg_callbacklistinfo *gci, const char *dtval)
+{
+	char buf[20];
+
+	const char *t;
+	struct tm tmbuf;
+	time_t secs;
+
+	if (strlen(dtval) == 10 && strchr(dtval, '-'))
+		return append_key(gci, dtval); /* YYYY-MM-DD */
+
+	t=strchr(dtval, 'T');
+
+	if (t && (t-dtval) == 8) /* YYYYMMDDThhmmss */
+	{
+		sprintf(buf, "%.4s-%.2s-%.2s", dtval, dtval+4, dtval+6);
+		return append_key(gci, buf);
+	}
+
+	secs=0;
+	while (dtval)
+	{
+		if (*dtval < '0' || *dtval > '9')
+			break;
+		secs = secs * 10 + (*dtval++ - '0');
+	}
+
+	if (secs == 0 || *dtval || localtime_r(&secs,  &tmbuf) == NULL)
+		return append_key(gci, "          ");
+
+	buf[strftime(buf, sizeof(buf)-1, "%Y-%m-%d", &tmbuf)]=0;
+
+	return append_key(gci, buf);
+}
+
 static int append_key(struct gpg_callbacklistinfo *gci, const char *l)
 {
 	int ll=strlen(l);
@@ -504,7 +541,7 @@ int libmail_gpg_listgroups(const char *gpgdir,
 		if (*p == 0)
 			continue;
 
-		q=unicode_cfromutf8(voidarg->charset, p, NULL);
+		q=libmail_u_convert_fromutf8(p, voidarg->charset, NULL);
 
 		if (!q)
 			continue;
