@@ -29,7 +29,6 @@
 #include	"maildir/maildirquota.h"
 #include	"maildir/maildiraclt.h"
 
-static const char rcsid[]="$Id: fetch.c,v 1.30 2010/03/19 01:09:26 mrsam Exp $";
 #if SMAP
 extern int smapflag;
 #endif
@@ -1488,13 +1487,8 @@ unsigned long l;
 static struct rfc2045 *cached_rfc2045p;
 static char *cached_filename;
 
-struct rfc2045 *fetch_alloc_rfc2045(unsigned long msgnum, FILE *fp)
+void fetch_free_cached()
 {
-	if (cached_rfc2045p &&
-	    strcmp(cached_filename,
-		   current_maildir_info.msgs[msgnum].filename) == 0)
-		return (cached_rfc2045p);
-
 	if (cached_rfc2045p)
 	{
 		rfc2045_free(cached_rfc2045p);
@@ -1502,6 +1496,16 @@ struct rfc2045 *fetch_alloc_rfc2045(unsigned long msgnum, FILE *fp)
 		free(cached_filename);
 		cached_filename=0;
 	}
+}
+
+struct rfc2045 *fetch_alloc_rfc2045(unsigned long msgnum, FILE *fp)
+{
+	if (cached_rfc2045p &&
+	    strcmp(cached_filename,
+		   current_maildir_info.msgs[msgnum].filename) == 0)
+		return (cached_rfc2045p);
+
+	fetch_free_cached();
 
 	if ((cached_filename=strdup(current_maildir_info.
 				    msgs[msgnum].filename))
