@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2007 Double Precision, Inc.
+** Copyright 1998 - 2011 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -59,6 +59,17 @@
 #include	"imapwrite.h"
 #include	"imapd.h"
 
+/*
+** RFC 2060: "A good value to use for the unique identifier validity value is a
+** 32-bit representation of the creation date/time of the mailbox."
+**
+** Well, Y2038k is on the horizon, time to push to reset button.
+**
+*/
+
+#ifndef IMAP_EPOCH
+#define IMAP_EPOCH	1000000000
+#endif
 
 static int do_imapscan_maildir2(struct imapscaninfo *, const char *,
 				int, int, struct uidplus_info *);
@@ -457,13 +468,13 @@ int	dowritecache=0;
 			return (-1);
 			*/
 		}
-		uidv=stat_buf.st_mtime;
+		uidv=stat_buf.st_mtime - IMAP_EPOCH;
 		dowritecache=1;
 	}
 	else
 	{
 		nextuid=1;
-		uidv=time(0);
+		uidv=time(0) - IMAP_EPOCH;
 	}
 
 	while (uidplus)
