@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2008 Double Precision, Inc.
+** Copyright 1998 - 2009 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -19,6 +19,7 @@
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	"rfc822/rfc822.h"
+#include	"rfc822/rfc822hdr.h"
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045/rfc2045.h"
 #include	"unicode/unicode.h"
@@ -28,7 +29,6 @@
 #include	"imaptoken.h"
 #include	"imapscanclient.h"
 
-static const char rcsid[]="$Id: search.c,v 1.33 2008/09/20 12:48:29 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern struct imapscaninfo current_maildir_info;
@@ -48,23 +48,16 @@ static void fill_search_quick(struct searchinfo *,
 	unsigned long, struct stat *);
 
 static void fill_search_header(struct searchinfo *,
-			       const struct unicode_info *,
+			       const char *,
 			       struct rfc2045 *, FILE *,
 			       struct imapscanmessageinfo *);
 
 static void fill_search_body(struct searchinfo *,
-			     const struct unicode_info *,
-	struct rfc2045 *, FILE *, struct imapscanmessageinfo *);
+			     struct rfc2045 *, FILE *,
+			     struct imapscanmessageinfo *);
 
 static int search_evaluate(struct searchinfo *);
 
-static void searcherror(const char *errmsg,
-                struct imapscanmessageinfo *mi)
-{
-        fprintf(stderr, "IMAP SEARCH ERROR: %s, uid=%u, filename=%s\n",
-                errmsg, (unsigned)getuid(), mi->filename);
-}
-
 static void search_callback(struct searchinfo *, struct searchinfo *, int,
 	unsigned long, void *);
 
@@ -78,9 +71,9 @@ static void search_callback(struct searchinfo *, struct searchinfo *, int,
 */
 
 void dosearch(struct searchinfo *si, struct searchinfo *sihead,
-	      const struct unicode_info *mycharset, int isuid)
+	      const char *charset, int isuid)
 {
-	search_internal(si, sihead, mycharset, isuid, search_callback, 0);
+	search_internal(si, sihead, charset, isuid, search_callback, 0);
 }
 
 static void search_callback(struct searchinfo *si, struct searchinfo *sihead,
@@ -93,7 +86,7 @@ static void search_callback(struct searchinfo *si, struct searchinfo *sihead,
 static void search_oneatatime(struct searchinfo *si,
 			      unsigned long i,
 			      struct searchinfo *sihead,
-			      const struct unicode_info *mycharset, int isuid,
+			      const char *charset, int isuid,
 			      void (*callback_func)(struct searchinfo *,
 						    struct searchinfo *, int,
 						    unsigned long, void *),
@@ -102,14 +95,14 @@ static void search_oneatatime(struct searchinfo *si,
 static void search_byKeyword(struct searchinfo *tree,
 			   struct searchinfo *keyword,
 			   struct searchinfo *sihead,
-			   const struct unicode_info *mycharset, int isuid,
+			   const char *charset, int isuid,
 			   void (*callback_func)(struct searchinfo *,
 						 struct searchinfo *, int,
 						 unsigned long, void *),
 			   void *voidarg);
 
 void search_internal(struct searchinfo *si, struct searchinfo *sihead,
-		     const struct unicode_info *mycharset, int isuid,
+		     const char *charset, int isuid,
 		     void (*callback_func)(struct searchinfo *,
 					   struct searchinfo *, int,
 					   unsigned long, void *),
@@ -124,22 +117,22 @@ void search_internal(struct searchinfo *si, struct searchinfo *sihead,
 	/* Shortcuts for keyword-based searches */
 
 	if (si->type == search_msgkeyword && si->bs == NULL && si->ke)
-		search_byKeyword(NULL, si, sihead, mycharset, isuid,
+		search_byKeyword(NULL, si, sihead, charset, isuid,
 			       callback_func, voidarg);
 	else if (si->type == search_and &&
 		 si->a->type == search_msgkeyword && si->a->bs == NULL
 		 && si->a->ke)
-		search_byKeyword(si->b, si->a, sihead, mycharset, isuid,
+		search_byKeyword(si->b, si->a, sihead, charset, isuid,
 			       callback_func, voidarg);
 	else for (i=0; i<current_maildir_info.nmessages; i++)
-		search_oneatatime(si, i, sihead, mycharset, isuid,
+		search_oneatatime(si, i, sihead, charset, isuid,
 				  callback_func, voidarg);
 }
 
 static void search_byKeyword(struct searchinfo *tree,
 			     struct searchinfo *keyword,
 			     struct searchinfo *sihead,
-			     const struct unicode_info *mycharset, int isuid,
+			     const char *charset, int isuid,
 			     void (*callback_func)(struct searchinfo *,
 						   struct searchinfo *, int,
 						   unsigned long, void *),
@@ -156,7 +149,7 @@ static void search_byKeyword(struct searchinfo *tree,
 			continue;
 		}
 
-		search_oneatatime(tree, n, sihead, mycharset, isuid,
+		search_oneatatime(tree, n, sihead, charset, isuid,
 				  callback_func, voidarg);
 	}
 }
@@ -168,7 +161,7 @@ static void search_byKeyword(struct searchinfo *tree,
 static void search_oneatatime(struct searchinfo *si,
 			      unsigned long i,
 			      struct searchinfo *sihead,
-			      const struct unicode_info *mycharset, int isuid,
+			      const char *charset, int isuid,
 			      void (*callback_func)(struct searchinfo *,
 						    struct searchinfo *, int,
 						    unsigned long, void *),
@@ -224,8 +217,8 @@ int	rc;
                         /* struct        rfc2045 *rfcp=rfc2045_fromfp(fp); */
                         struct        rfc2045 *rfcp=rfc2045header_fromfp(fp); 
 
-			fill_search_header(sihead, mycharset, rfcp, fp,
-				current_maildir_info.msgs+i);
+			fill_search_header(sihead, charset, rfcp, fp,
+					   current_maildir_info.msgs+i);
 			rc=search_evaluate(si);
                         rfc2045_free(rfcp); 
 
@@ -234,8 +227,8 @@ int	rc;
 				/* Ok, search message contents */
                                 struct        rfc2045 *rfcp=rfc2045_fromfp(fp);
 
-				fill_search_body(sihead, mycharset, rfcp, fp,
-					current_maildir_info.msgs+i);
+				fill_search_body(sihead, rfcp, fp,
+						 current_maildir_info.msgs+i);
 
 				/*
 				** If there are still UNKNOWN nodes, change
@@ -634,17 +627,28 @@ int	rc, rc2;
 
 /* ------- header search -------- */
 
-static void fill_search_header_recursive(struct searchinfo *,
-					 const struct unicode_info *,
-	struct rfc2045 *, FILE *, struct imapscanmessageinfo *);
+struct fill_search_header_info {
+
+	struct searchinfo *si;
+
+	char *utf8buf;
+	size_t utf8buflen;
+	size_t utf8bufsize;
+};
+
+static int headerfilter_func(const char *name, const char *value, void *arg);
+static int fill_search_header_utf8(const char *, size_t, void *);
+static int fill_search_header_done(const char *, void *);
 
 static void fill_search_header(struct searchinfo *si,
-			       const struct unicode_info *mycharset,
-	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
+			       const char *charset,
+			       struct rfc2045 *rfcp, FILE *fp,
+			       struct imapscanmessageinfo *mi)
 {
-	off_t start_pos, end_pos, start_body;
-	off_t nlines, nbodylines;
 	struct searchinfo *sip;
+	struct rfc2045src *src;
+	struct rfc2045_decodemsgtoutf8_cb decodecb;
+	struct fill_search_header_info decodeinfo;
 
 	/* Consider the following dummy nodes as evaluated */
 
@@ -655,131 +659,151 @@ static void fill_search_header(struct searchinfo *si,
 		case search_references2:
 		case search_references3:
 		case search_references4:
+		case search_cc:
+		case search_date:
+		case search_from:
+		case search_to:
 			sip->value=1;
 			break;
 		default:
 			break;
 		}
 
-	rfc2045_mimepos(rfcp, &start_pos, &end_pos, &start_body,
-		&nlines, &nbodylines);
+	search_set_charset_conv(si, charset);
 
-	/*
-	** Since headers can contain any character set, we set the search
-	** strings to their native character set.
-	*/
+	src=rfc2045src_init_fd(fileno(fp));
 
-	search_set_charset_conv(si, mycharset, mycharset);
-
-	if (fseek(fp, start_pos, SEEK_SET) < 0)
-	{
-		searcherror("fseek failed", mi);
+	if (!src)
 		return;
-	}
 
-	while (start_pos < start_body)
+	memset(&decodecb, 0, sizeof(decodecb));
+	memset(&decodeinfo, 0, sizeof(decodeinfo));
+
+	decodeinfo.si=si;
+
+	decodecb.flags=RFC2045_DECODEMSG_NOBODY
+		| RFC2045_DECODEMSG_NOHEADERNAME;
+	decodecb.headerfilter_func=headerfilter_func;
+	decodecb.output_func=fill_search_header_utf8;
+	decodecb.headerdone_func=fill_search_header_done;
+	decodecb.arg=&decodeinfo;
+
+	rfc2045_decodemsgtoutf8(src, rfcp, &decodecb);
+	rfc2045src_deinit(src);
+	if (decodeinfo.utf8buf)
+		free(decodeinfo.utf8buf);
+}
+
+static int headerfilter_func(const char *name, const char *value, void *arg)
+{
+	struct fill_search_header_info *decodeinfo=
+		(struct fill_search_header_info *)arg;
+	struct searchinfo *sip;
+	const char *p;
+	int isto=rfc822hdr_namecmp(name, "to");
+	int iscc=rfc822hdr_namecmp(name, "cc");
+	int isfrom=rfc822hdr_namecmp(name, "from");
+	int isinreplyto=rfc822hdr_namecmp(name, "in-reply-to");
+	int isdate=rfc822hdr_namecmp(name, "date");
+
+	int isreferences=rfc822hdr_namecmp(name, "references");
+	int ismessageid=rfc822hdr_namecmp(name, "message-id");
+
+	for (sip=decodeinfo->si; sip; sip=sip->next)
 	{
-	char	*p=readline(0, fp);
-	char	*q=0;
-	char	*r;
-	int	isdate;
-	char *conv_buf;
-	char *conv_buf_uc;
-	const struct unicode_info *search_chset;
-
-		/* Collect multiline header into the readline buffer */
-
-		while (p)
+		if (sip->type == search_text && sip->value <= 0)
 		{
-		int	c;
+			/*
+			** Full message search. Reset the search engine,
+			** feed it "Headername: "
+			*/
 
-			start_pos=ftell(fp);
-			if (start_pos >= start_body)	break;
+			maildir_search_reset(&sip->sei);
 
-			c=getc(fp);
-			if (c != EOF)	ungetc(c, fp);
-			if (c == '\n' || !isspace((int)(unsigned char)c))
-				break;
-			p=readline(strlen(p), fp);
-		}
-		if (!p)	break;
-
-		/* search_text applies to headers too */
-
-		for (r=p; *r && *r != ':'; r++)
-			*r=toupper(*r);
-
-		if (r)
-			*r++=0;
-
-		while (*r && isspace((int)(unsigned char)*r))
-			++r;
-
-		search_chset=mycharset;
-
-		if (search_chset->search_chset)
-			search_chset=search_chset->search_chset;
-
-		conv_buf=rfc2047_decode_unicode(r, search_chset,
-						RFC2047_DECODE_DISCARD);
-
-			/* Characters not in our charset are discarded */
-
-		if (!conv_buf && errno != EINVAL)
-			write_error_exit(0);
-
-		conv_buf_uc= (*search_chset->toupper_func)(search_chset,
-							   conv_buf, 0);
-		if (!conv_buf_uc)
-			write_error_exit(0);
-
-		for (sip=si; sip; sip=sip->next)
-		{
-			if (sip->type == search_text &&
-			    sip->bs &&
-			    sip->value <= 0)
+			for (p=name; *p; p++)
 			{
-				maildir_search_reset(&sip->sei);
-
-				for (q=conv_buf_uc; *q; q++)
-					maildir_search_step(&sip->sei,
-						    (unsigned char)*q);
+				maildir_search_step_unicode_lc(&sip->sei,
+							       (unsigned char)
+							       *p);
+				if (maildir_search_found(&sip->sei))
+					sip->value=1;
+			}
+			for (p=": "; *p; p++)
+			{
+				maildir_search_step_unicode_lc(&sip->sei,
+							       (unsigned char)
+							       *p);
 				if (maildir_search_found(&sip->sei))
 					sip->value=1;
 			}
 		}
 
-		q=conv_buf_uc;
-
-		isdate=strcmp(p, "DATE");
-		for (sip=si; sip; sip=sip->next)
+		if ( (sip->type == search_cc && iscc == 0 && sip->as == 0)
+		     ||
+		     (sip->type == search_from && isfrom == 0 && sip->as == 0)
+		     ||
+		     (sip->type == search_to && isto == 0 && sip->as == 0)
+		     ||
+		     (sip->type == search_references1 && isinreplyto == 0
+		      && sip->bs == 0))
 		{
-			if (sip->type == search_text)
+			struct rfc822t *t;
+			struct rfc822a *a;
+			char *s;
+
+			t=rfc822t_alloc_new(value, NULL, NULL);
+			if (!t) write_error_exit(0);
+			a=rfc822a_alloc(t);
+			if (!a) write_error_exit(0);
+			s=a->naddrs > 0 ? rfc822_getaddr(a, 0):strdup("");
+			rfc822a_free(a);
+			rfc822t_free(t);
+			if (!s) write_error_exit(0);
+
+			if (sip->type == search_references1)
 			{
-				if (sip->value <= 0 &&
-				    sip->bs && maildir_search_found(&sip->sei))
-					sip->value=1;
+				sip->bs=malloc(strlen(s)+3);
+				if (!sip->bs)
+					write_error_exit(0);
+				strcat(strcat(strcpy(sip->bs, "<"), s), ">");
+				free(s);
 			}
+			else
+				sip->as=s;
+		}
 
-			if (isdate == 0) switch (sip->type)	{
-			time_t given_time, msg_time;
+		switch (sip->type) {
+		case search_orderedsubj:
 
-			case search_orderedsubj:
-				if (sip->bs == 0)
-				{
-					sip->bs=strdup(q);
-					if (!sip->bs)
-						write_error_exit(0);
-				}
+			if (isdate == 0 && sip->bs == 0)
+			{
+				sip->bs=strdup(value);
+				if (!sip->bs)
+					write_error_exit(0);
+			}
+			break;
+
+		case search_date:
+
+			if (isdate == 0 && sip->as == 0)
+			{
+				time_t msg_time=rfc822_parsedt(value);
+
+				sip->as=timestamp_for_sorting(msg_time);
+			}
+			break;
+
+		case search_sentbefore:
+		case search_sentsince:
+		case search_senton:
+
+			if (sip->value > 0)
 				break;
-			case search_sentbefore:
-			case search_sentsince:
-			case search_senton:
 
-				if (sip->value > 0)	break;
-
-				given_time=decode_date(sip->as);
-				msg_time=rfc822_parsedt(q);
+			if (isdate == 0)
+			{
+				time_t given_time=decode_date(sip->as);
+				time_t msg_time=rfc822_parsedt(value);
 
 				if (given_time == 0 || msg_time == 0)
 					break;
@@ -793,361 +817,250 @@ static void fill_search_header(struct searchinfo *si,
 					(sip->type == search_senton &&
 						msg_time == given_time))
 					sip->value=1;
-				break;
-			default:
-				break;
 			}
-
-			if (sip->type == search_cc && strcmp(p, "CC") == 0 &&
-				sip->as == 0)
-			{
-			struct rfc822t *t=rfc822t_alloc_new(q, NULL, NULL);
-			struct rfc822a *a=t ? rfc822a_alloc(t):0;
-
-				sip->as= a && a->naddrs > 0 ?
-					rfc822_getaddr(a, 0):0;
-				if (a)	rfc822a_free(a);
-				if (t)	rfc822t_free(t);
-			}
-
-			if (sip->type == search_date && strcmp(p, "DATE") == 0
-				&& sip->as == 0)
-			{
-			time_t msg_time=rfc822_parsedt(q);
-
-				sip->as=timestamp_for_sorting(msg_time);
-			}
-
-			if (sip->type == search_from && strcmp(p, "FROM") == 0
-				&& sip->as == 0)
-			{
-			struct rfc822t *t=rfc822t_alloc_new(q, NULL, NULL);
-			struct rfc822a *a=t ? rfc822a_alloc(t):0;
-
-				sip->as= a && a->naddrs > 0 ?
-					rfc822_getaddr(a, 0):0;
-				if (a)	rfc822a_free(a);
-				if (t)	rfc822t_free(t);
-			}
-
-			if (sip->type == search_to && strcmp(p, "TO") == 0 &&
-				sip->as == 0)
-			{
-			struct rfc822t *t=rfc822t_alloc_new(q, NULL, NULL);
-			struct rfc822a *a=t ? rfc822a_alloc(t):0;
-
-				sip->as= a && a->naddrs > 0 ?
-					rfc822_getaddr(a, 0):0;
-				if (a)	rfc822a_free(a);
-				if (t)	rfc822t_free(t);
-			}
-
-			switch (sip->type)	{
-			case search_cc:
-			case search_date:
-			case search_from:
-			case search_to:
-				sip->value=1;
-				break;
-			default:
-				break;
-			}
-
-			if (sip->type == search_references1 &&
-			    strcmp(p, "REFERENCES") == 0 && sip->as == 0)
-			{
-				sip->as=strdup(conv_buf);
-				if (!sip->as)
-					write_error_exit(0);
-			}
-
-			if (sip->type == search_references1 &&
-			    strcmp(p, "IN-REPLY-TO") == 0 && sip->bs == 0)
-			{
-				/* Extract first address from In-REPLY-To */
-
-				struct rfc822t *t;
-				struct rfc822a *a;
-				char *s;
-
-				t=rfc822t_alloc_new(conv_buf, NULL, NULL);
-				if (!t) write_error_exit(0);
-				a=rfc822a_alloc(t);
-				if (!a) write_error_exit(0);
-				s=a->naddrs > 0 ? rfc822_getaddr(a, 0):
-					strdup("");
-				rfc822a_free(a);
-				rfc822t_free(t);
-				if (!s) write_error_exit(0);
-
-				sip->bs=malloc(strlen(s)+3);
-				if (!sip->bs)
-					write_error_exit(0);
-				strcat(strcat(strcpy(sip->bs, "<"), s), ">");
-				free(s);
-			}
-
-			if (sip->type == search_references2 &&
-			    strcmp(p, "DATE") == 0 && sip->as == 0)
-			{
-				sip->as=strdup(q);
-				if (!sip->as)
-					write_error_exit(0);
-			}
-
-			if (sip->type == search_references3 &&
-			    strcmp(p, "SUBJECT") == 0 && sip->as == 0)
-			{
-				sip->as=strdup(q);
-				if (!sip->as)
-					write_error_exit(0);
-			}
-
-			if (sip->type == search_references4 &&
-			    strcmp(p, "MESSAGE-ID") == 0 && sip->as == 0)
-			{
-				sip->as=strdup(conv_buf);
-				if (!sip->as)
-					write_error_exit(0);
-			}
-
-			if (sip->type == search_orderedsubj &&
-				strcmp(p, "SUBJECT") == 0 && sip->as == 0)
-			{
-				int dummy;
-				char *r;
-
-			/* Dummy node used to suck in the Subject:
-			** header for the THREAD or SORT command.
-			*/
-				r=rfc2047_decode_unicode(q, &unicode_UTF8, 0);
-				if (!r)
-					write_error_exit(0);
-
-				sip->as=rfc822_coresubj(r, &dummy);
-				free(r);
-				if (!sip->as)
-					write_error_exit(0);
-			}
-
-			if (sip->type != search_header ||
-			    sip->bs == 0 ||
-				sip->value > 0)	continue;
-
-			if (strcmp(sip->cs, p) == 0)
-			{
-				sip->value=0;
-				maildir_search_reset(&sip->sei);
-				for (r=q; *r; r++)
-					maildir_search_step(&sip->sei, *r);
-				if (maildir_search_found(&sip->sei))
-					sip->value=1;
-			}
-		}
-		free(conv_buf_uc);
-		free(conv_buf);
-	}
-	fill_search_header_recursive(si, mycharset, rfcp, fp, mi);
-}
-
-/* Scan the headers of attached messages too! */
-
-static void fill_search_header_recursive(struct searchinfo *si,
-					 const struct unicode_info *mycharset,
-	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
-{
-	if (!rfcp)	return;
-
-	if (rfcp->firstpart && rfcp->firstpart->next == 0 &&
-		!rfcp->firstpart->isdummy)
-	{
-		fill_search_header(si, mycharset, rfcp->firstpart, fp, mi);
-		return;
-	}
-
-	rfcp=rfcp->firstpart;
-
-	while (rfcp)
-	{
-		if (!rfcp->isdummy)
-			fill_search_header_recursive(si, mycharset,
-						     rfcp, fp, mi);
-		rfcp=rfcp->next;
-	}
-}
-
-static int search_body_func(const char *, size_t, void *);
-
-/* Last straw - search message contents */
-
-struct search_linebuf {
-	char buf[BUFSIZ];
-	int l;
-	const struct unicode_info *uc;
-	struct searchinfo *si;
-} ;
-
-static void fill_search_body(struct searchinfo *si,
-			     const struct unicode_info *mycharset,
-	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
-{
-struct searchinfo *sip;
-off_t start_pos, end_pos, start_body;
-off_t nlines, nbodylines;
-const char *content_type_s;
-const char *content_transfer_encoding_s;
-const char *charset_s;
-char	buf[BUFSIZ];
-int	rc;
-
- struct search_linebuf slb;
-
-	for (sip=si; sip; sip=sip->next)
-		if ((sip->type == search_text || sip->type == search_body)
-			&& sip->value <= 0)
 			break;
 
-	if (sip == 0)	return;	/* Nothing to search any more */
-
-	if (rfcp->isdummy)	return;	/* stub RFC2045 section */
-
-	if (rfcp->firstpart)	/* Search multiline contents */
-	{
-		for (rfcp=rfcp->firstpart; rfcp; rfcp=rfcp->next)
-			fill_search_body(si, mycharset, rfcp, fp, mi);
-		return;
-	}
-
-	/*
-	** Since headers can contain any character set, we set the search
-	** strings to their native character set.
-	*/
-
-	rfc2045_mimeinfo(rfcp, &content_type_s,
-		&content_transfer_encoding_s, &charset_s);
-	rfc2045_mimepos(rfcp, &start_pos, &end_pos, &start_body,
-		&nlines, &nbodylines);
-
-	slb.l=0;
-	slb.uc=unicode_find(charset_s);
-	slb.si=si;
-
-	if (!slb.uc)
-		return;	/* Don't know how to search this charset */
-
-	search_set_charset_conv(si, mycharset, slb.uc);
-
-	if (fseek(fp, start_body, SEEK_SET) < 0)
-	{
-		searcherror("fseek failed", mi);
-		return;
-	}
-	rfc2045_cdecode_start(rfcp, search_body_func, &slb);
-
-	for (sip=si; sip; sip=sip->next)
-		if ((sip->type == search_text || sip->type == search_body)
-		    && sip->bs
-		    && sip->value <= 0)
-		{
-			si->value=0;
-			maildir_search_reset(&si->sei);
-		}
-
-	rc=0;
-	while (start_body < end_pos)
-	{
-	unsigned n=sizeof(buf);
-
-		if (n > end_pos - start_body)
-			n=end_pos - start_body;
-
-		rc=fread(buf, 1, n, fp);
-		if (rc <= 0)
-		{
-			searcherror("fread failed", mi);
+		case search_references1:
+			if (isreferences == 0 && sip->as == 0)
+			{
+				sip->as=strdup(value);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+			break;
+		case search_references2:
+			if (isdate == 0 && sip->as == 0)
+			{
+				sip->as=strdup(value);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+			break;
+		case search_references4:
+			if (ismessageid == 0 && sip->as == 0)
+			{
+				sip->as=strdup(value);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+			break;
+		default:
 			break;
 		}
-		start_body += rc;
-		rc=rfc2045_cdecode(rfcp, buf, rc);
-		if (rc)	break;
 	}
-
-	if (rc == 0)	(void)rfc2045_cdecode_end(rfcp);
-	for (sip=si; sip; sip=sip->next)
-		if ((sip->type == search_text || sip->type == search_body)
-		    && sip->bs
-		    && sip->value <= 0)
-		{
-			if (maildir_search_found(&sip->sei))
-				sip->value=1;
-			else
-				sip->value=0;
-		}
+	decodeinfo->utf8buflen=0;
+	return 1;
 }
 
-static int search_body_func(const char *ptr, size_t cnt, void *voidptr)
+static int fill_search_header_utf8(const char *str, size_t cnt, void *arg)
 {
-	struct search_linebuf *slb=(struct search_linebuf *)voidptr;
+	struct fill_search_header_info *decodeinfo=
+		(struct fill_search_header_info *)arg;
 
-	struct searchinfo *si, *sihead=(struct searchinfo *)slb->si;
-	size_t	i, j;
-
-	for (i=0; i<cnt; i++)
+	if (decodeinfo->utf8bufsize - decodeinfo->utf8buflen < cnt)
 	{
-		char c=ptr[i];
-		char *p;
-		const struct unicode_info *uc_info;
+		size_t newsize=decodeinfo->utf8buflen + cnt*2;
+		char *p=decodeinfo->utf8buf
+			? realloc(decodeinfo->utf8buf, newsize):
+			malloc(newsize);
 
-		if (c != '\n' && c != '\r')
-		{
-			if (slb->l < sizeof(slb->buf)-2)
-				slb->buf[slb->l++]=c;
-			continue;
-		}
-		slb->buf[slb->l]=0;
-
-		uc_info=slb->uc;
-
-		if (uc_info->search_chset)
-		{
-			char *q;
-
-			p=unicode_xconvert(slb->buf, uc_info,
-					   uc_info->search_chset);
-
-			if (p)
-			{
-				uc_info=uc_info->search_chset;
-				q=(uc_info->toupper_func)(uc_info, p, 0);
-				free(p);
-				p=q;
-			}
-
-		}
-		else
-			p=(*uc_info->toupper_func)(uc_info, slb->buf, 0);
 		if (!p)
 			write_error_exit(0);
+		decodeinfo->utf8buf=p;
+		decodeinfo->utf8bufsize=newsize;
+	}
 
-		for (si=sihead; si; si=si->next)
-		{
-			if ((si->type == search_text ||
-			     si->type == search_body)
-			    && si->bs && si->value <= 0)
+	if (cnt)
+		memcpy(decodeinfo->utf8buf+decodeinfo->utf8buflen, str, cnt);
+	decodeinfo->utf8buflen += cnt;
+	return 0;
+}
+
+static int fill_search_header_done(const char *name, void *arg)
+{
+	struct fill_search_header_info *decodeinfo=
+		(struct fill_search_header_info *)arg;
+	struct searchinfo *sip;
+	int issubject=rfc822hdr_namecmp(name, "subject");
+	size_t j;
+	libmail_u_convert_handle_t conv;
+	unicode_char *ucptr;
+	size_t ucsize;
+	int rc;
+
+	if (decodeinfo->utf8buflen &&
+	    decodeinfo->utf8buf[decodeinfo->utf8buflen-1] == '\n')
+		--decodeinfo->utf8buflen;
+
+	fill_search_header_utf8("", 1, arg);
+
+	for (sip=decodeinfo->si; sip; sip=sip->next)
+		switch (sip->type) {
+		case search_references3:
+			if (issubject == 0 && sip->as == 0)
 			{
-				si->value=0;
-				maildir_search_reset(&si->sei);
+				sip->as=strdup(decodeinfo->utf8buf);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+			break;
+		case search_orderedsubj:
 
-				for (j=0; p[j]; j++)
-					maildir_search_step(&si->sei,
-						    (unsigned char)p[j]);
+			if (issubject == 0 && sip->as == 0)
+			{
+				int dummy;
 
-				if (maildir_search_found(&si->sei))
-					si->value=1;
+				sip->as=rfc822_coresubj(decodeinfo->utf8buf,
+							&dummy);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+			break;
+		case search_header:
+
+			if (sip->cs == NULL || rfc822hdr_namecmp(sip->cs, name))
+				break;
+
+			/* FALLTHRU */
+
+		case search_text:
+			if (sip->value > 0)
+				break;
+
+			maildir_search_reset(&sip->sei);
+
+			conv=libmail_u_convert_tou_init("utf-8", &ucptr,
+							&ucsize, 0);
+
+			if (!conv)
+				break;
+
+			rc=libmail_u_convert(conv, decodeinfo->utf8buf,
+					     decodeinfo->utf8buflen-1);
+
+			if (libmail_u_convert_deinit(conv, NULL))
+				break;
+
+			if (rc)
+			{
+				free(ucptr);
+				break;
+			}
+
+			for (j=0; j<=ucsize; ++j)
+			{
+				maildir_search_step_unicode_lc(&sip->sei,
+							       j == ucsize
+							       ? ' ':
+							       ucptr[j]);
+				if (maildir_search_found(&sip->sei))
+				{
+					sip->value=1;
+					break;
+				}
+			}
+			free(ucptr);
+			break;
+		default:
+			break;
+		}
+
+
+	return 0;
+}
+
+struct fill_search_body_info {
+
+	struct searchinfo *si;
+	libmail_u_convert_handle_t toucs4_handle;
+
+};
+
+static int fill_search_body_utf8(const char *str, size_t n, void *arg);
+static int fill_search_body_ucs4(const char *str, size_t n, void *arg);
+
+static void fill_search_body(struct searchinfo *si,
+			     struct rfc2045 *rfcp, FILE *fp,
+			     struct imapscanmessageinfo *mi)
+{
+	struct rfc2045src *src;
+	struct rfc2045_decodemsgtoutf8_cb decodecb;
+	struct fill_search_body_info decodeinfo;
+	struct searchinfo *sip;
+
+	src=rfc2045src_init_fd(fileno(fp));
+
+	if (!src)
+		return;
+
+	memset(&decodecb, 0, sizeof(decodecb));
+	memset(&decodeinfo, 0, sizeof(decodeinfo));
+
+	decodecb.flags=RFC2045_DECODEMSG_NOHEADERS;
+	decodecb.output_func=fill_search_body_utf8;
+	decodecb.arg=&decodeinfo;
+
+	decodeinfo.si=si;
+
+	if ((decodeinfo.toucs4_handle=
+	     libmail_u_convert_init("utf-8",
+				    libmail_u_ucs4_native,
+				    fill_search_body_ucs4,
+				    &decodeinfo)) == NULL)
+	{
+		write_error_exit("libmail_u_convert_init");
+	}
+
+	for (sip=decodeinfo.si; sip; sip=sip->next)
+		if ((sip->type == search_text || sip->type == search_body)
+		    && sip->value <= 0)
+		{
+			rfc2045_decodemsgtoutf8(src, rfcp, &decodecb);
+			break;
+		}
+
+	libmail_u_convert_deinit(decodeinfo.toucs4_handle, NULL);
+
+	rfc2045src_deinit(src);
+}
+
+static int fill_search_body_utf8(const char *str, size_t n, void *arg)
+{
+	struct fill_search_body_info *decodeinfo=
+		(struct fill_search_body_info *)arg;
+
+	return libmail_u_convert(decodeinfo->toucs4_handle, str, n);
+}
+
+static int fill_search_body_ucs4(const char *str, size_t n, void *arg)
+{
+	struct fill_search_body_info *decodeinfo=
+		(struct fill_search_body_info *)arg;
+	struct searchinfo *sip;
+	const unicode_char *u=(const unicode_char *)str;
+	int notfound=1;
+
+	n /= 4;
+
+	for (sip=decodeinfo->si; sip; sip=sip->next)
+		if ((sip->type == search_text || sip->type == search_body)
+		    && sip->value <= 0)
+		{
+			size_t i;
+
+			notfound=0;
+
+			for (i=0; i<n; i++)
+			{
+				maildir_search_step_unicode_lc(&sip->sei, u[i]);
+
+				if (maildir_search_found(&sip->sei))
+				{
+					sip->value=1;
+					break;
+				}
 			}
 		}
-		free(p);
-		slb->l=0;
-	}
-	return (0);
+
+	return notfound;
 }
