@@ -3,7 +3,6 @@
 ** See COPYING for distribution information.
 */
 
-static const char rcsid[]="$Id: thread.c,v 1.15 2009/11/08 18:14:47 mrsam Exp $";
 
 #include	"config.h"
 #include	"imapd.h"
@@ -157,12 +156,12 @@ static void printos(struct os_threadinfo **array, size_t cnt)
 }
 
 void dothreadorderedsubj(struct searchinfo *si, struct searchinfo *sihead,
-			 const struct unicode_info *mycharset, int isuid)
+			 const char *charset, int isuid)
 {
 struct	os_struct	os;
 
 	os_init(&os);
-	search_internal(si, sihead, mycharset, isuid, thread_os_callback, &os);
+	search_internal(si, sihead, charset, isuid, thread_os_callback, &os);
 
 	if (os.nmsgs > 0)	/* Found some messages */
 	{
@@ -209,7 +208,7 @@ static void thread_os_callback(struct searchinfo *si,
 static void printthread(struct imap_refmsg *, int);
 
 void dothreadreferences(struct searchinfo *si, struct searchinfo *sihead,
-			const struct unicode_info *mycharset,
+			const char *charset,
 			int isuid)
 {
 	struct imap_refmsgtable *reftable;
@@ -221,7 +220,7 @@ void dothreadreferences(struct searchinfo *si, struct searchinfo *sihead,
 		return;
 	}
 
-	search_internal(si, sihead, mycharset, 0,
+	search_internal(si, sihead, charset, 0,
 			thread_ref_callback, reftable);
 
 	root=rfc822_thread(reftable);
@@ -383,7 +382,7 @@ size_t	i;
 }
 
 void dosortmsgs(struct searchinfo *si, struct searchinfo *sihead,
-		const struct unicode_info *mycharset, int isuid)
+		const char *charset, int isuid)
 {
 struct	sortmsgs sm;
 struct	searchinfo *p;
@@ -403,7 +402,7 @@ struct	searchinfo *p;
 		default:
 			break;
 		}
-	search_internal(si, sihead, mycharset, isuid, sort_callback, &sm);
+	search_internal(si, sihead, charset, isuid, sort_callback, &sm);
 	if (sm.nmsgs > 0)
 	{
 	size_t	i;
