@@ -586,7 +586,6 @@ static void list(int argn, int argc, char **argv, int flags)
 {
 	struct listinfo listinfo;
 	int all_flag=0;
-	int showevent=0;
 	struct PCP *pcp;
 	struct PCP_list_all list_all;
 
@@ -611,7 +610,6 @@ static void list(int argn, int argc, char **argv, int flags)
 			     listinfo.list_to=
 			     parse_datetime(&argn, argc, argv)) == 0)
 				usage();
-			showevent=1;
 			continue;
 		}
 
@@ -1225,27 +1223,25 @@ static int list_msg_mime(struct rfc2045 *rfc, FILE *fp)
 
         rfc2045_mimeinfo(rfc, &content_type, &dummy, &dummy);
 
-	if (rfc2231_udecodeDisposition(rfc, "name", NULL, &disposition_name)<0
-	    ||
-	    rfc2231_udecodeDisposition(rfc, "filename", NULL,
-				       &disposition_filename) < 0
-	    ||
-	    rfc2231_udecodeType(rfc, "name", NULL,
-				&content_name) < 0)
-	{
-		perror("malloc");
-		exit(1);
-	}
+	if (rfc2231_udecodeDisposition(rfc, "name", NULL, &disposition_name)<0)
+		disposition_name=NULL;
+
+	if (rfc2231_udecodeDisposition(rfc, "filename", NULL,
+				       &disposition_filename) < 0)
+		disposition_filename=NULL;
+
+	if (rfc2231_udecodeType(rfc, "name", NULL, &content_name) < 0)
+		content_name=NULL;
 
         rfc2045_mimepos(rfc, &start_pos, &end_pos, &start_body,
                         &dummy2, &dummy2);
 
 	disposition_filename_s=disposition_filename;
 
-	if (!*disposition_filename_s)
-		disposition_filename_s=content_name;
 	if (!disposition_filename_s || !*disposition_filename_s)
 		disposition_filename_s=disposition_name;
+	if (!disposition_filename_s || !*disposition_filename_s)
+		disposition_filename_s=content_name;
 
 	printf(gettext("Attachment: %s (%s)\n"), content_type,
 	       libmail_str_sizekb(end_pos - start_body, buffer));
@@ -1253,9 +1249,10 @@ static int list_msg_mime(struct rfc2045 *rfc, FILE *fp)
 		printf("    %s\n", disposition_filename_s);
 	printf("\n");
 
-	free(content_name);
-	free(disposition_name);
-	free(disposition_filename);
+	if (content_name)		free(content_name);
+	if (disposition_name)		free(disposition_name);
+	if (disposition_filename)	free(disposition_filename);
+
 	return (0);
 }
 
