@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2006 Double Precision, Inc.
+** Copyright 1998 - 2009 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -52,7 +52,6 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.44 2006/11/04 20:09:47 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *accesslocal=0;
@@ -65,6 +64,7 @@ static const char *maxprocsarg=0;
 static const char *warnarg=0;
 static const char *maxperiparg=0;
 static const char *maxpercarg=0;
+static const char *droparg=0;
 static const char *nodnslookup=0;
 static const char *noidentlookup=0;
 static const char *stderrarg=0;
@@ -83,6 +83,7 @@ static struct args arginfo[]={
 	{"access", &accessarg},
 	{"accesslocal", &accesslocal},
 	{"denymsg", &denymsgarg},
+	{"drop", &droparg},
 	{"address", &ipaddrarg},
 	{"block", 0, setup_block},
 	{"group", &grouparg},
@@ -1051,6 +1052,7 @@ static int doit(int argn, int argc, char **argv)
 	}
 #endif
 
+	signal(SIGPIPE, SIG_IGN);
 	for (;;)
 	{
 		int n;
@@ -1094,6 +1096,20 @@ static int doit(int argn, int argc, char **argv)
 	}
 }
 
+static void denied(int sockfd)
+{
+	if (denymsgarg) {
+		if (write(sockfd, denymsgarg, strlen(denymsgarg)) < 0 ||
+		    write(sockfd, "\n", 1) < 0)
+		{
+			sox_close(sockfd);
+			_exit(1);
+		}
+	}
+	sox_close(sockfd);
+	_exit(0);
+}
+
 static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 		     const char *prog,
 		     char **args)
@@ -1172,7 +1188,7 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 	{
 		wait_restore(childsig);
 
-		if (accesslocal) // Lookup local interface address too?
+		if (accesslocal) /* Lookup local interface address too? */
 		{
 		RFC1035_NETADDR lsin;
 		RFC1035_ADDR laddr;
@@ -1191,16 +1207,7 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 
 		if (allowaccess(&addr,0) == 0)
 		{
-			if (denymsgarg) {
-				if (write(sockfd, denymsgarg, strlen(denymsgarg)) < 0 ||
-				    write(sockfd, "\n", 1) < 0)
-				{
-					sox_close(sockfd);
-					_exit(1);
-				}
-			}
-			sox_close(sockfd);
-			_exit(0);
+			denied(sockfd);
 		}
 
 		run(sockfd, &addr, addrport, prog, args);
@@ -1264,23 +1271,29 @@ static void ip2host(const RFC1035_ADDR *addr, const char *env)
 const char *remotehost="softdnserr";
 char	buf[RFC1035_MAXNAMESIZE+1];
 
+#if TCPDUSERFC1035
+struct rfc1035_res res;
+#endif
+
 	if (nodnslookup)	return;
 
 	rfc1035_ntoa(addr, buf);
 
 #if TCPDUSERFC1035
 
-	if (rfc1035_ptr(&rfc1035_default_resolver, addr, buf) != 0)
+	rfc1035_init_resolv(&res);
+
+	if (rfc1035_ptr(&res, addr, buf) != 0)
 	{
 		if (errno == ENOENT)
 			remotehost=0;
 	}
 	else
 	{
-	RFC1035_ADDR *ias;
-	unsigned nias, n;
+		RFC1035_ADDR *ias;
+		unsigned nias, n;
 
-		if (rfc1035_a(&rfc1035_default_resolver, buf, &ias, &nias) != 0)
+		if (rfc1035_a(&res, buf, &ias, &nias) != 0)
 		{
 			if (errno == ENOENT)
 				remotehost=0;
@@ -1303,6 +1316,8 @@ char	buf[RFC1035_MAXNAMESIZE+1];
 			}
 		}
 	}
+	rfc1035_destroy_resolv(&res);
+
 #else
 
 	{
@@ -1401,14 +1416,15 @@ char *r;
 
 static void docheckblocklist(struct blocklist_s *p, const char *nameptr)
 {
-const char *q;
-const char *varname=p->var;
-char	hostname[RFC1035_MAXNAMESIZE+1];
-char	buf[RFC1035_MAXNAMESIZE+1];
-int	hasnotxt;
-struct blocklist_s *pp;
-struct rfc1035_reply *replyp;
-int i;
+	const char *q;
+	const char *varname=p->var;
+	char	hostname[RFC1035_MAXNAMESIZE+1];
+	char	buf[RFC1035_MAXNAMESIZE+1];
+	int	hasnotxt;
+	struct blocklist_s *pp;
+	struct rfc1035_reply *replyp;
+	struct rfc1035_res res;
+	int i;
 
 	hostname[0]=0;
 	strncat(hostname, nameptr, RFC1035_MAXNAMESIZE);
@@ -1418,6 +1434,8 @@ int i;
 	if ((q=getenv(varname)) != 0)	return;
 					/* Env var already set */
 
+	rfc1035_init_resolv(&res);
+
 	if (p->ia.s_addr == INADDR_ANY)
 	{
 		/* We're not looking for a particular token IP address */
@@ -1428,8 +1446,7 @@ int i;
 			** We don't have a predefined error message, therefore
 			** we expect TXT records.
 			*/
-			if ((i=rfc1035_resolve_cname(&rfc1035_default_resolver,
-					RFC1035_RESOLVE_RECURSIVE,
+			if ((i=rfc1035_resolve_cname(&res,
 					hostname,
 					RFC1035_TYPE_TXT,
 					RFC1035_CLASS_IN, &replyp, 0)) >= 0)
@@ -1448,8 +1465,7 @@ int i;
 			** for A records only.
 			*/
 
-			if ((i=rfc1035_resolve_cname(&rfc1035_default_resolver,
-					RFC1035_RESOLVE_RECURSIVE,
+			if ((i=rfc1035_resolve_cname(&res,
 					hostname,
 					RFC1035_TYPE_A,
 					RFC1035_CLASS_IN, &replyp, 0)) >= 0)
@@ -1459,6 +1475,7 @@ int i;
 		}
 
 		if (replyp)	rfc1035_replyfree(replyp);
+		rfc1035_destroy_resolv(&res);
 		return;
 	}
 
@@ -1482,13 +1499,16 @@ int i;
 	** issue an ANY query, and parse the results.
 	*/
 
-	(void)rfc1035_resolve_cname(&rfc1035_default_resolver,
-			RFC1035_RESOLVE_RECURSIVE,
+	(void)rfc1035_resolve_cname(&res,
 			hostname,
 			hasnotxt ? RFC1035_TYPE_ANY:RFC1035_TYPE_A,
 			RFC1035_CLASS_IN, &replyp, 0);
 
-	if (!replyp)	return;
+	if (!replyp)
+	{
+		rfc1035_destroy_resolv(&res);
+		return;
+	}
 
 	for (i=0; i<replyp->ancount+replyp->nscount+replyp->arcount; i++)
 	{
@@ -1531,7 +1551,7 @@ int i;
 
 			/* No predefined message, look for a TXT record. */
 
-			if ((j=rfc1035_replysearch_all(&rfc1035_default_resolver,
+			if ((j=rfc1035_replysearch_all(&res,
 						       replyp, hostname,
 						       RFC1035_TYPE_TXT,
 						       RFC1035_CLASS_IN, 0)) >= 0)
@@ -1543,6 +1563,7 @@ int i;
 		}
 	}
 	rfc1035_replyfree(replyp);
+	rfc1035_destroy_resolv(&res);
 }
 
 static void check_blocklist_ipv4(struct blocklist_s *p,
@@ -1583,6 +1604,52 @@ static void check_blocklist(struct blocklist_s *p, const RFC1035_ADDR *ia)
 }
 #endif
 
+static void check_drop(int sockfd)
+{
+	const char *p, *q;
+	char *r;
+
+	p=droparg;
+
+	if (p && !*p)
+		p="BLOCK";
+
+	for (; p && *p; q=p)
+	{
+		if (*p == ',')
+		{
+			q= ++p;
+			continue;
+		}
+
+		for (q=p; *q; ++q)
+			if (*q == ',')
+				break;
+
+		r=malloc(q-p+1);
+
+		if (!r)
+		{
+			perror("malloc");
+			_exit(1);
+		}
+
+		memcpy(r, p, q-p);
+		r[q-p]=0;
+
+		p=getenv(r);
+		free(r);
+
+		if (p && *p)
+		{
+			fprintf(stderr,
+				"WARN: dropped blocked connection from %s\n",
+				getenv("TCPREMOTEIP"));
+			denied(sockfd);
+		}
+	}
+}
+
 static void proxy();
 
 static void run(int fd, const RFC1035_ADDR *addr, int addrport,
@@ -1675,6 +1742,7 @@ const char *p;
 	for (bl=blocklist; bl; bl=bl->next)
 		check_blocklist(bl, addr);
 
+	check_drop(fd);
 	sox_close(0);
 	sox_close(1);
 	sox_dup(fd);
@@ -1686,6 +1754,8 @@ const char *p;
 		sox_dup(1);
 	}
 	proxy();
+	signal(SIGPIPE, SIG_DFL);
+
 	execv(prog, argv);
 	perror(prog);
 	exit(1);
