@@ -1,10 +1,10 @@
+#include "config.h"
 /*
-** Copyright 2000-2009 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 /*
-** $Id: addressbook.c,v 1.17 2009/11/22 14:42:56 mrsam Exp $
 */
 
 #include	"sqwebmail.h"
@@ -20,6 +20,9 @@
 
 #define	ADDRESSBOOK	"sqwebmail-addressbook"
 
+#define TOUTF8(what)	\
+	libmail_u_convert_toutf8((what), sqwebmail_content_charset, NULL)
+
 extern const char *sqwebmail_content_charset;
 
 extern void output_attrencoded(const char *);
@@ -206,18 +209,13 @@ static void ab_add_int(const char *name, const char *address, const char *nick)
 void ab_add(const char *name, const char *address, const char *nick)
 {
 	char *nick_utf8, *name_utf8, *addr_utf8;
-	const struct unicode_info *uiptr
-		=unicode_find(sqwebmail_content_charset);
 
 	if (*nick == 0 || *address == 0)
 		return;
 
-	if (!uiptr)
-		uiptr=&unicode_ISO8859_1;
-
-	nick_utf8=unicode_convert(nick, uiptr, &unicode_UTF8);
-	name_utf8=unicode_convert(name ? name:"", uiptr, &unicode_UTF8);
-	addr_utf8=unicode_convert(address, uiptr, &unicode_UTF8);
+	nick_utf8=TOUTF8(nick);
+	name_utf8=TOUTF8(name ? name:"");
+	addr_utf8=TOUTF8(address);
 
 	if (nick_utf8 && name_utf8 && addr_utf8)
 		ab_add_int(name_utf8, addr_utf8, nick_utf8);
@@ -425,11 +423,6 @@ void ab_listselect_fp(FILE *w)
 	char *header, *value;
 	struct	abooklist *a=0, *b, **aa;
 	size_t	acnt=0, i;
-	const struct unicode_info *uiptr
-		=unicode_find(sqwebmail_content_charset);
-
-	if (!uiptr)
-		uiptr=&unicode_ISO8859_1;
 
 	if ((fp=fopen(ADDRESSBOOK, "r")) != 0)
 	{
@@ -460,11 +453,9 @@ void ab_listselect_fp(FILE *w)
 
 		for (i=0; i<acnt; i++)
 		{
-			char *p;
-
-			p=unicode_xconvert(aa[i]->name,
-					   &unicode_UTF8,
-					   uiptr);
+			char *p=libmail_u_convert_fromutf8(aa[i]->name,
+							   sqwebmail_content_charset,
+							   NULL);
 
 			fprintf(w, "<option value=\"");
 			output_attrencoded_fp(p ? p:aa[i]->name, w);
@@ -585,14 +576,21 @@ static int ab_addrselect_cb(const char *a, const char *n, void *vp)
 	return (0);
 }
 
-static void ab_show_utf8(const char *p,
-			 const struct unicode_info *info)
+static void ab_show_utf8(const char *p)
 {
-	char *p_s=unicode_xconvert(p, &unicode_UTF8, info);
+	int err;
+	char *p_s=libmail_u_convert_fromutf8(p, sqwebmail_content_charset,
+					     &err);
 
 	if (!p_s)
 		return;
 
+	if (err)
+	{
+		free(p_s);
+		return;
+	}
+
 	p=p_s;
 
 	while (*p)
@@ -624,21 +622,16 @@ static void ab_show_utf8(const char *p,
 
 void ab_nameaddr_show(const char *name, const char *addr)
 {
-	const struct unicode_info *info=unicode_find(sqwebmail_content_charset);
-
-	if (!info)
-		return;
-
 	if (name)
 	{
 		printf("\"");
-		ab_show_utf8(name, info);
+		ab_show_utf8(name);
 		printf("\"&nbsp;");
 	}
 	printf("&lt;");
 
 	if (addr)
-		ab_show_utf8(addr, info);
+		ab_show_utf8(addr);
 
 	printf("&gt;");
 }
@@ -714,12 +707,6 @@ char	*edit_name=0;
 char	*edit_addr=0;
 int	replace_index=0;
 
-	const struct unicode_info *uiptr
-		=unicode_find(sqwebmail_content_charset);
-
-	if (!uiptr)
-		uiptr=&unicode_ISO8859_1;
-
 #if 0
 	fp=fopen("/tmp/pid", "w");
 	fprintf(fp, "%d", getpid());
@@ -787,9 +774,7 @@ int	replace_index=0;
 
 	if (*cgi("nick.delete"))
 	{
-		char *p=unicode_xconvert(cgi("nick"),
-					 uiptr,
-					 &unicode_UTF8);
+		char *p=TOUTF8(cgi("nick"));
 
 		do_edit=0;
 
@@ -810,10 +795,7 @@ int	replace_index=0;
 		{
 			if ((fp=fopen(ADDRESSBOOK, "r")) != 0)
 			{
-				char *editnick_utf8=
-					unicode_xconvert(editnick,
-							 uiptr,
-							 &unicode_UTF8);
+				char *editnick_utf8=TOUTF8(editnick);
 
 				while ((header=maildir_readheader_nolc(fp,
 					&value)) != NULL)
@@ -832,14 +814,11 @@ int	replace_index=0;
 					if (a)
 					{
 						char *newname_utf8=
-							unicode_xconvert(newname,
-									 uiptr,
-									 &unicode_UTF8);
+							TOUTF8(newname);
+
 						char *newaddr_utf8=
-							unicode_xconvert(newaddr,
-									 uiptr,
-									 &unicode_UTF8);
-						
+							TOUTF8(newaddr);
+
 						dodel(editnick_utf8, a,
 						      atoi(replacenum),
 						      newname_utf8 && newaddr_utf8 ?
@@ -899,8 +878,7 @@ int	replace_index=0;
 
 		if ((fp=fopen(ADDRESSBOOK, "r")) != 0)
 		{
-			char *s_utf8=unicode_xconvert(s, uiptr,
-						      &unicode_UTF8);
+			char *s_utf8=TOUTF8(s);
 
 			while ((header=maildir_readheader_nolc(fp, &value))
 				!= NULL)
@@ -979,7 +957,7 @@ int	replace_index=0;
 							if (n)
 							{
 								printf("\"");
-								ab_show_utf8(n, uiptr);
+								ab_show_utf8(n);
 								printf("\"");
 								free(n);
 							}
@@ -993,7 +971,7 @@ int	replace_index=0;
 
 						if (s)
 						{
-							ab_show_utf8(s, uiptr);
+							ab_show_utf8(s);
 							free(s);
 						}
 						printf("&gt;</td><td><input type=\"submit\" name=\"startedit%d\" value=\"%s\" />&nbsp;<input type=\"submit\" name=\"del%d\" value=\"%s\" /></td></tr>\n",
@@ -1012,16 +990,20 @@ int	replace_index=0;
 
 		if (edit_name)
 		{
+			int err;
 			char *edit_name_native=
-				unicode_xconvert(edit_name,
-						 &unicode_UTF8,
-						 uiptr);
+				libmail_u_convert_fromutf8(edit_name,
+							   sqwebmail_content_charset,
+							   &err);
 
 			if (edit_name_native)
 			{
-				printf(" value=\"");
-				output_attrencoded(edit_name_native);
-				printf("\"");
+				if (err == 0)
+				{
+					printf(" value=\"");
+					output_attrencoded(edit_name_native);
+					printf("\"");
+				}
 				free(edit_name_native);
 			}
 		}
@@ -1030,15 +1012,21 @@ int	replace_index=0;
 		printf("<tr><td align=\"right\">%s</td><td><input type=\"text\" name=\"newaddress\" class=\"nicknewaddr\"", nick_address);
 		if (edit_addr)
 		{
+			int err;
 			char *edit_addr_native=
-				unicode_xconvert(edit_addr,
-						 &unicode_UTF8,
-						 uiptr);
+				libmail_u_convert_fromutf8(edit_addr,
+							   sqwebmail_content_charset,
+							   &err);
+
 			if (edit_addr_native)
 			{
-				printf(" value=\"");
-				output_attrencoded(edit_addr_native);
-				printf("\"");
+				if (err == 0)
+				{
+					printf(" value=\"");
+					output_attrencoded(edit_addr_native);
+					printf("\"");
+				}
+				free(edit_addr_native);
 			}
 		}
 
