@@ -904,7 +904,11 @@ int attach_upload(const char *draft,
 	if (attpubkey || attprivkey)
 	{
 		argvec[2]="application/pgp-keys";
-		n=3;
+		argvec[3]="-N";
+		argvec[4]="pgpkeys.txt";
+		argvec[5]="-a";
+		argvec[6]="Content-Disposition: attachment; filename=\"pgpkeys.txt\"";
+		n=7;
 		filenamebuf=0;
 	}
 	else
