@@ -5,7 +5,6 @@
 
 
 /*
-** $Id: auth.c,v 1.45 2009/06/27 16:32:38 mrsam Exp $
 */
 
 #include	"sqwebmail.h"
@@ -253,7 +252,6 @@ static char *hdrbuf=0;
 
 FILE *fp;
 char authcharset[128];
-const struct unicode_info *uiptr=NULL, *uoptr=NULL;
 char *ufullname=0;
 static char *uhdrbuf=0;
 
@@ -275,13 +273,12 @@ static char *uhdrbuf=0;
 	if (authcharset[0] == 0
 	    && sqwebmail_system_charset && *sqwebmail_system_charset
 	    && strcasecmp(sqwebmail_system_charset, "ASCII"))
-		strncpy(authcharset, sqwebmail_system_charset,
+		strncat(authcharset, sqwebmail_system_charset,
 			sizeof(authcharset)-1);
 
-	if (authcharset[0] && (uiptr=unicode_find(authcharset))
+	if (authcharset[0]
 	    && sqwebmail_content_charset && *sqwebmail_content_charset
-	    && (uoptr=unicode_find(sqwebmail_content_charset))
-	    && (ufullname=unicode_ctoutf8(uiptr, fullname, NULL)))
+	    && (ufullname=libmail_u_convert_toutf8(fullname, authcharset,NULL)))
 		fullname = ufullname;
 
 	l=sizeof("\"\" <>")+strlen(address)+strlen(fullname);
@@ -319,8 +316,11 @@ static char *uhdrbuf=0;
 
 	if (ufullname)	free(ufullname);
 	if (uhdrbuf)	free(uhdrbuf);
-	if (uoptr && (uhdrbuf=unicode_cfromutf8(uoptr, hdrbuf, NULL)))
+	if ((uhdrbuf=libmail_u_convert_fromutf8(hdrbuf,
+						sqwebmail_content_charset,
+						NULL)) != NULL)
 		return (uhdrbuf);
+
 	return (hdrbuf);
 }
 
