@@ -1,9 +1,8 @@
 /*
-** $Id: autoresponse.c,v 1.21 2008/07/20 17:00:33 mrsam Exp $
 */
 
 /*
-** Copyright 2001-2008 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -18,7 +17,7 @@
 #include	"maildir/maildirmisc.h"
 #include	"maildir/maildirfilter.h"
 #include	"rfc2045/rfc2045.h"
-#include	"rfc2045/rfc2646.h"
+#include	"newmsg.h"
 #include	"cgi/cgi.h"
 #include	"numlib/numlib.h"
 #include	<string.h>
@@ -43,24 +42,23 @@ extern const char *calc_mime_type(const char *filename);
 
 extern void charset_warning(const char *);
 
-static int save_autoresponse(const char *p, size_t l, void *vp)
+static void save_autoresponse(const char *p, size_t l, void *vp)
 {
 	FILE *fp=*(FILE **)vp;
 
 	if (fp)
 		if (fwrite(p, l, 1, fp) != 1)
 			; /* ignore */
-	return (0);
 }
 
-static int display_autoresponse(struct rfc2646parser *, int, void *);
-
-struct parseinfo {
-	int last_was_flowed;
-} ;
-
 static int read_headers(FILE *);
 
+static int show_autoresponse_trampoline(const char *ptr, size_t cnt, void *arg)
+{
+	show_textarea((struct show_textarea_info *)arg, ptr, cnt);
+	return 0;
+}
+
 void autoresponse()
 {
 const char	*autoresp_title1=getarg("TITLE1");
@@ -109,11 +107,7 @@ const char	*autoresp_text2=getarg("TEXT2");
 	{
 		const char *autorespname=cgi("autoresponse_choose");
 		FILE *fp;
-		struct rfc2646parser *parser;
-		char buf[BUFSIZ];
-		int i;
 		char *s=folder_fromutf7(autorespname);
-		struct parseinfo info;
 		const char *pp;
 
 		if (!s)
@@ -140,18 +134,6 @@ const char	*autoresp_text2=getarg("TEXT2");
 			return;
 		}
 
-		info.last_was_flowed=0;
-
-		if ((parser=rfc2646_alloc( &display_autoresponse, &info))
-		    == NULL)
-		{
-			free(s);
-			if (fp)
-				fclose(fp);
-			printf(getarg("ERROR"), strerror(errno));
-			return;
-		}
-
 		printf("<input type=\"hidden\" name=\"autoresponse\" value=\"");
 		output_attrencoded(autorespname);
 		printf("\" />\n");
@@ -162,13 +144,30 @@ const char	*autoresp_text2=getarg("TEXT2");
 		if (pp && *pp)
 			output_attrencoded(pp);
 		else
-			while ((i=fread(buf, 1, sizeof(buf), fp)) > 0)
+		{
+			struct show_textarea_info info;
+			libmail_u_convert_handle_t h;
+
+			show_textarea_init(&info, 0);
+
+			h=libmail_u_convert_init("utf-8",
+						 sqwebmail_content_charset,
+						 show_autoresponse_trampoline,
+						 &info);
+
+			if (h)
 			{
-				if (rfc2646_parse(parser, buf, i))
-					break;
+				size_t i;
+				char buf[BUFSIZ];
+
+				while ((i=fread(buf, 1, sizeof(buf), fp)) > 0)
+				{
+					libmail_u_convert(h, buf, i);
+				}
+				libmail_u_convert_deinit(h, NULL);
 			}
+		}
 
-		rfc2646_free(parser);
 		if (fp)
 			fclose(fp);
 		printf("%s\n", autoresp_text2);
@@ -215,34 +214,10 @@ static int read_headers(FILE *fp)
 		return (-1);
 	}
 
-	charset_warning(charset);
 	rfc2045_free(rfc2045p);
 	return (0);
 }
 
-static int display_autoresponse(struct rfc2646parser *lineinfo,
-				int isflowed, void *vp)
-{
-	struct parseinfo *info=(struct parseinfo *)vp;
-
-	if (!isflowed && lineinfo->line[0] == 0 &&
-	    info->last_was_flowed)
-	{
-		printf("\n");
-		info->last_was_flowed=0;
-		return (0);
-	}
-
-	if (info->last_was_flowed)
-		printf(" ");
-	else
-		printf("\n");
-
-	output_attrencoded(lineinfo->line);
-	info->last_was_flowed=isflowed;
-	return (0);
-}
-
 static FILE *upload_attachment(const char *);
 
 void autoresponsedelete()
@@ -252,26 +227,16 @@ void autoresponsedelete()
 		const char *autorespname=cgi("autoresponse");
 		const char *autoresptxt=cgi("text");
 		FILE *fp;
-		struct rfc2646create *create_ptr;
 		size_t l;
-		const struct unicode_info *charset=
-			unicode_find(sqwebmail_content_charset);
-
-		if (!charset)
-			charset=&unicode_ISO8859_1;
 
 		if ((fp=upload_attachment(autorespname)) == NULL)
 		{
-			if ((create_ptr=
-			     rfc2646create_alloc(&save_autoresponse,
-						 charset, &fp))
-			    == NULL ||
-			    (fp=maildir_autoresponse_create(NULL,
+			struct wrap_info uw;
+
+			if ((fp=maildir_autoresponse_create(NULL,
 							    autorespname))
 			    == NULL)
 			{
-				if (create_ptr)
-					rfc2646create_free(create_ptr);
 				printf(getarg("SAVEFAILED"), strerror(errno));
 				return;
 			}
@@ -282,15 +247,12 @@ void autoresponsedelete()
 				--l;
 
 			fprintf(fp, "Content-Type: text/plain");
-			fprintf(fp, "; charset=\"%s\"",
-				sqwebmail_content_charset);
-			fprintf(fp, "; format=flowed");
-			fprintf(fp, "\n");
+			fprintf(fp, "; format=flowed; delsp=yes"
+				"; charset=\"utf-8\"\n");
 			fprintf(fp, "Content-Transfer-Encoding: 8bit\n\n");
 
-			rfc2646create_parse(create_ptr, autoresptxt, l);
-			rfc2646create_parse(create_ptr, "\n\n", 2);
-			rfc2646create_free(create_ptr);
+			wrap_text_init(&uw, "utf-8", save_autoresponse, &fp);
+			wrap_text(&uw, autoresptxt, l);
 		}
 
 		if (fflush(fp) || ferror(fp))
