@@ -162,6 +162,7 @@ void folder_contents_title()
 const char *lab;
 const char *f;
 const char *inbox_lab, *drafts_lab, *trash_lab, *sent_lab;
+int in_utf8;
 
 	lab=getarg("FOLDERTITLE");
 
@@ -174,10 +175,13 @@ const char *inbox_lab, *drafts_lab, *trash_lab, *sent_lab;
 	sent_lab=getarg("SENT");
 
 	f=sqwebmail_folder;
+	in_utf8=1;
+
 	if (strcmp(f, INBOX) == 0)	f=inbox_lab;
 	else if (strcmp(f, INBOX "." DRAFTS) == 0)	f=drafts_lab;
 	else if (strcmp(f, INBOX "." SENT) == 0)	f=sent_lab;
 	else if (strcmp(f, INBOX "." TRASH) == 0)	f=trash_lab;
+	else in_utf8=0;
 
 	if (lab)
 	{
@@ -185,7 +189,11 @@ const char *inbox_lab, *drafts_lab, *trash_lab, *sent_lab;
 
 		printf("%s", lab);
 
-		origff=ff=folder_fromutf7(f);
+		origff=ff=in_utf8 ?
+			libmail_u_convert_fromutf8(f,
+						   sqwebmail_content_charset,
+						   NULL)
+			: folder_fromutf7(f);
 
 		if (strcmp(ff, NEWSHAREDSP) == 0 ||
 		    strncmp(ff, NEWSHAREDSP ".", sizeof(NEWSHAREDSP)) == 0)
@@ -423,7 +431,7 @@ MATCHEDSTR **matches;
 int	i, found;
 int	morebefore, moreafter;
 const char	*nomsg, *selectalllab;
-const char	*unselectalllab, *golab;
+ const char	*unselectalllab;
 const char	*qerrmsg;
 long highend;
 unsigned long last_message_searched=0;
@@ -488,7 +496,6 @@ unsigned long *last_message_searched_ptr=NULL;
 
 	time(&current_time);
 	nomsg=getarg("NOMESSAGES");
-	golab=getarg("GO");
 	selectalllab=getarg("SELECTALL");
 	unselectalllab=getarg("UNSELECTALL");
 
@@ -746,11 +753,6 @@ static void do_folder_navigate(const char *dir, size_t pos, long highend,
 
 	if (morebefore)
 	{
-	size_t	beforepos;
-
-		if (pos < pref_flagpagesize)	beforepos=0;
-		else	beforepos=pos-pref_flagpagesize;
-
 		printf("<a href=\"");
 		output_scriptptrget();
 		printf("&amp;form=folder&amp;pos=0\" style=\"text-decoration: none\">");
@@ -3279,7 +3281,6 @@ static void do_folderlist(const char *inbox_pfix,
 
 		size_t	j;
 		const char *pfix;
-		int isshared=0;
 		int isunsubscribed=0;
 		const char	*img=folder_img;
 
@@ -3291,7 +3292,6 @@ static void do_folderlist(const char *inbox_pfix,
 			char	*dir;
 			struct	stat	stat_buf;
 
-			isshared=1;
 			pfix="+++";
 
 			dir=maildir_shareddir(".",
@@ -3600,7 +3600,8 @@ void folder_download(const char *folder, size_t pos, const char *mimeid)
 	free(filename);
 
 	cginocache();
-	msg2html_download(fp, mimeid, *cgi("download") == '1');
+	msg2html_download(fp, mimeid, *cgi("download") == '1',
+			  sqwebmail_content_charset);
 
 	fclose(fp);
 }
