@@ -1,10 +1,10 @@
+#include "config.h"
 /*
 ** Copyright 2000-2006 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 /*
-** $Id: ldaplist.c,v 1.15 2008/07/20 03:15:08 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	<stdio.h>
@@ -147,9 +147,6 @@ const char	*sysbook=getarg("SYSBOOK");
 
 static char *getfilter()
 {
-	const struct unicode_info *uoptr =
-		unicode_find(sqwebmail_content_charset);
-
 	const char *p;
 
 	if (!*cgi("ldapsearch"))
@@ -160,10 +157,7 @@ static char *getfilter()
 	if (!*p)
 		return NULL;
 
-
-	return unicode_ctoutf8(uoptr, p, NULL);
-
-	return strdup(p);
+	return libmail_u_convert_toutf8(p, sqwebmail_content_charset, NULL);
 }
 
 struct search_info {
@@ -202,12 +196,10 @@ static int parsesearch(const char *cn, const char *mail,
 	char	numbuf[NUMBUFSIZE];
 	char	numbuf2[NUMBUFSIZE+10];
 
-	char *cn_native=NULL;
+	char *cn_native;
 
-	const struct unicode_info *uiptr=
-		unicode_find(sqwebmail_content_charset);
-
-	cn_native=unicode_cfromutf8(uiptr, cn, NULL);
+	cn_native=libmail_u_convert_fromutf8(cn, sqwebmail_content_charset,
+					     NULL);
 
 	if (cn_native)
 		cn=cn_native;
