@@ -1,11 +1,10 @@
 /*
-** Copyright 1998 - 2010 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: maildir.c,v 1.94 2010/03/19 01:09:26 mrsam Exp $
 */
 #include	"config.h"
 #include	<stdio.h>
@@ -19,6 +18,7 @@
 #include	"folder.h"
 #include	"rfc822/rfc822.h"
 #include	"rfc822/rfc2047.h"
+#include	"rfc2045/rfc2045.h"
 #include	"cgi/cgi.h"
 #include	"pref.h"
 #include	"sqconfig.h"
@@ -32,6 +32,7 @@
 #include	"maildir/maildircreate.h"
 #include	"maildir/maildirinfo.h"
 #include	"maildir/maildiraclt.h"
+#include	"maildir/maildirsearch.h"
 #include	"htmllibdir.h"
 
 #if	HAVE_UNISTD_H
@@ -62,9 +63,7 @@
 #include	<utime.h>
 #endif
 
-#if UTF7_FOLDER_ENCODING
 #include	"unicode/unicode.h"
-#endif
 
 #include	"strftime.h"
 
@@ -412,7 +411,6 @@ static char *foldercachename(const char *folder)
 
 static int opencache(const char *folder, const char *mode)
 {
-char	*maildir=xlate_shmdir(folder);
 char	*cachename;
 size_t	l;
 char	*p;
@@ -420,7 +418,12 @@ char	*q;
 char	*r;
 unsigned long ul;
 
-	if (!maildir)	return (-1);
+	{
+		char	*maildir=xlate_shmdir(folder);
+
+		if (!maildir)	return (-1);
+		free(maildir);
+	}
 
 	cachename=foldercachename(folder);
 
@@ -1473,6 +1476,32 @@ void maildir_purgemimegpg()
 		closedir(dirp);
 }
 
+/* Ditto for search results */
+
+void maildir_purgesearch()
+{
+	DIR	*dirp;
+	struct	dirent	*dire;
+	char *p;
+
+	for (dirp=opendir("tmp"); dirp && (dire=readdir(dirp)) != 0; )
+	{
+		if (strstr(dire->d_name, ":search:") == 0)	continue;
+
+		p=malloc(sizeof("tmp/")+strlen(dire->d_name));
+
+		if (p)
+		{
+			strcat(strcpy(p, "tmp/"), dire->d_name);
+			unlink(p);
+			free(p);
+		}
+	}
+
+	if (dirp)
+		closedir(dirp);
+}
+
 static int subjectcmp(const char *a, const char *b)
 {
 	int	aisre;
@@ -1481,13 +1510,13 @@ static int subjectcmp(const char *a, const char *b)
 	char	*as;
 	char	*bs;
 
-	as=rfc822_display_hdrvalue_tobuf("subject", a, unicode_UTF8.chset,
+	as=rfc822_display_hdrvalue_tobuf("subject", a, "utf-8",
 					 NULL, NULL);
 
 	if (!as)
 		as=strdup(a);
 
-	bs=rfc822_display_hdrvalue_tobuf("subject", b, unicode_UTF8.chset,
+	bs=rfc822_display_hdrvalue_tobuf("subject", b, "utf-8",
 					 NULL, NULL);
 
 	if (!bs)
@@ -1633,8 +1662,8 @@ char	c;
 }
 
 MSGINFO **maildir_read(const char *dirname, unsigned nfiles,
-	size_t *starting_pos,
-	int *morebefore, int *moreafter)
+		       size_t *starting_pos,
+		       int *morebefore, int *moreafter)
 {
 MSGINFO	**msginfo;
 size_t	i;
@@ -1657,11 +1686,885 @@ size_t	i;
 		if (*starting_pos + i >= all_cnt)	break;
 		if ((msginfo[i]= get_msginfo_alloc(*starting_pos + i)) == 0)
 			break;
+
+		msginfo[i]->msgnum=*starting_pos+i;
 	}
 	*moreafter= *starting_pos + i < all_cnt;
 	return (msginfo);
 }
 
+#define save_int(n, fp) do {						\
+		size_t i;						\
+		size_t cnt=sizeof((n));					\
+		putc(cnt, (fp));					\
+		for (i=0; i<cnt; i++)					\
+			putc((n) >> (cnt-1-i)*8, (fp));			\
+	} while(0);
+
+static void save_str(const char *str, FILE *fp)
+{
+	size_t l=strlen(str);
+
+	save_int(l, fp);
+	fprintf(fp, "%s", str);
+}
+
+#define load_int(n, fp) do {					\
+	int i;							\
+	int cnt=getc(fp);					\
+	(n)=0;							\
+	if (cnt != EOF)						\
+		for (i=0; i<cnt; ++i)				\
+			(n)=(n) << 8 | (unsigned char)getc(fp);	\
+	} while(0);
+
+static char *load_str(FILE *fp)
+{
+	size_t l;
+	size_t i;
+	char *str;
+
+	load_int(l, fp);
+
+	if (feof(fp))
+		l=0;
+
+	str=malloc(l+1);
+
+	if (!str)
+		enomem();
+
+	for (i=0; i<l; i++)
+	{
+		char c=getc(fp);
+
+		str[i]=c;
+	}
+
+	str[i]=0;
+	return str;
+}
+
+static void save_msginfo(const MSGINFO *p, MATCHEDSTR *context, FILE *fp)
+{
+	size_t context_cnt;
+	MATCHEDSTR *c;
+
+	save_int(p->msgnum, fp);
+	save_str(p->filename, fp);
+	save_str(p->date_s, fp);
+	save_str(p->from_s, fp);
+	save_str(p->subject_s, fp);
+	save_str(p->size_s, fp);
+	save_int(p->date_n, fp);
+	save_int(p->size_n, fp);
+	save_int(p->mi_mtime, fp);
+	save_int(p->mi_ino, fp);
+
+	for (context_cnt=0, c=context; c && c->match; ++c, ++context_cnt)
+		;
+
+	save_int(context_cnt, fp);
+
+	for (c=context; c && c->match; ++c)
+	{
+		save_str(c->prefix, fp);
+		save_str(c->match, fp);
+		save_str(c->suffix, fp);
+	}
+}
+
+static void load_msginfo(MSGINFO **retinfo,
+			 MATCHEDSTR **retmatches,
+			 FILE *fp)
+{
+	MSGINFO *p=malloc(sizeof(MSGINFO));
+	size_t context_cnt;
+	MATCHEDSTR *c;
+
+	if (p == 0)
+		enomem();
+
+	load_int(p->msgnum, fp);
+	p->filename=load_str(fp);
+	p->date_s=load_str(fp);
+	p->from_s=load_str(fp);
+	p->subject_s=load_str(fp);
+	p->size_s=load_str(fp);
+	load_int(p->date_n, fp);
+	load_int(p->size_n, fp);
+	load_int(p->mi_mtime, fp);
+	load_int(p->mi_ino, fp);
+
+	*retinfo=p;
+
+	load_int(context_cnt, fp);
+
+	*retmatches=NULL;
+
+	if (context_cnt)
+	{
+		*retmatches=c=malloc(sizeof(MATCHEDSTR)*(context_cnt+1));
+
+		for (; context_cnt; --context_cnt)
+		{
+			char *prefix=load_str(fp);
+			char *match=load_str(fp);
+			char *suffix=load_str(fp);
+
+			if (c)
+			{
+				c->prefix=prefix;
+				c->match=match;
+				c->suffix=suffix;
+				++c;
+			}
+			else
+			{
+				free(prefix);
+				free(match);
+				free(suffix);
+			}
+		}
+
+		if (c)
+		{
+			c->prefix=NULL;
+			c->match=NULL;
+			c->suffix=NULL;
+		}
+	}
+}
+
+static void execute_maildir_search(const char *dirname,
+				   size_t pos,
+				   const char *searchtxt,
+				   const char *charset,
+				   unsigned nfiles,
+				   MSGINFO ***retval,
+				   MATCHEDSTR ***retcontext,
+				   unsigned long *last_message_searched);
+
+#define SEARCHFORMATVER 1
+
+void maildir_search(const char *dirname,
+		    size_t pos,
+		    const char *searchtxt,
+		    const char *charset,
+		    unsigned nfiles)
+{
+	struct maildir_tmpcreate_info createInfo;
+
+	MSGINFO **p;
+	MATCHEDSTR **pcontext;
+	char *filename;
+	FILE *fp;
+	unsigned i;
+
+	unsigned long last_message_searched=0;
+
+	execute_maildir_search(dirname, pos, searchtxt, charset,
+			       nfiles, &p, &pcontext, &last_message_searched);
+
+	maildir_purgesearch();
+
+	maildir_tmpcreate_init(&createInfo);
+
+	createInfo.uniq=":search:";
+	createInfo.doordie=1;
+
+	if ((fp=maildir_tmpcreate_fp(&createInfo)) == NULL)
+	{
+		if (p)
+			maildir_free(p, nfiles);
+		error("Can't create new file.");
+	}
+
+	filename=createInfo.tmpname;
+	createInfo.tmpname=NULL;
+	maildir_tmpcreate_free(&createInfo);
+
+	chmod(filename, 0600);
+
+	{
+		char ver=SEARCHFORMATVER;
+
+		save_int(ver, fp);
+		save_int(last_message_searched, fp);
+	}
+
+	for (i=0; i<nfiles; i++)
+		if (p[i])
+			save_msginfo(p[i], pcontext[i], fp);
+	fflush(fp);
+	if (ferror(fp) || fclose(fp))
+		error("Cannot create temp file");
+
+	cgi_put(SEARCHRESFILENAME, strrchr(filename, '/')+1);
+
+	if (p)
+		maildir_free(p, nfiles);
+	if (pcontext)
+		matches_free(pcontext, nfiles);
+}
+
+void maildir_loadsearch(unsigned nfiles,
+			MSGINFO ***retmsginfo,
+			MATCHEDSTR ***retmatches,
+			unsigned long *last_message_searched)
+{
+	MSGINFO	**msginfo;
+	MATCHEDSTR **matches;
+
+	unsigned i;
+	const char *filename;
+	char *buf;
+	FILE *fp;
+	char ver;
+
+	if ((msginfo=malloc(sizeof(MSGINFO *)*nfiles)) == 0)
+		enomem();
+
+	if ((matches=malloc(sizeof(MATCHEDSTR *)*nfiles)) == 0)
+	{
+		free(msginfo);
+		enomem();
+	}
+
+	for (i=0; i<nfiles; i++)
+	{
+		msginfo[i]=0;
+		matches[i]=0;
+	}
+
+	filename=cgi(SEARCHRESFILENAME);
+	CHECKFILENAME(filename);
+
+	buf=malloc(strlen(filename)+5);
+
+	if (!buf)
+	{
+		free(msginfo);
+		enomem();
+	}
+
+	strcat(strcpy(buf, "tmp/"), filename);
+
+	fp=fopen(buf, "r");
+	free(buf);
+
+	if (fp)
+	{
+		load_int(ver, fp);
+
+		if (ver == SEARCHFORMATVER)
+		{
+			load_int(*last_message_searched, fp);
+			for (i=0; i<nfiles; ++i)
+			{
+				int ch=getc(fp);
+
+				if (ch == EOF)
+					break;
+				ungetc(ch, fp);
+
+				load_msginfo(&msginfo[i], &matches[i], fp);
+			}
+		}
+	}
+	fclose(fp);
+
+	for (i=0; i<nfiles; ++i)
+	{
+		MSGINFO *p;
+
+		if (msginfo[i] == 0)
+			continue;
+
+		p=get_msginfo(msginfo[i]->msgnum);
+
+		if (p && p->mi_ino == msginfo[i]->mi_ino) /* Safety first */
+		{
+			char *f=strdup(p->filename);
+			if (f)
+			{
+				if (msginfo[i]->filename)
+					free(msginfo[i]->filename);
+				msginfo[i]->filename=f;
+			}
+		}
+	}
+
+	*retmsginfo=msginfo;
+	*retmatches=matches;
+}
+
+static const char spaces[]=" \t\r\n";
+
+#define SEARCH_MATCH_CONTEXT_LEN	20
+
+/* After matching, save surrounding context here */
+
+struct searchresults_match_context {
+	struct searchresults_match_context *next;
+
+	unicode_char *match_context_before;
+	unicode_char *match_context;
+
+	unicode_char *match_context_after;
+	size_t match_context_after_len;
+	size_t match_context_after_max_len;
+};
+
+struct searchresults {
+
+	char prevchar;
+	size_t foundcnt;
+
+	int finished;
+
+	char utf8buf[512];
+	size_t utf8buf_cnt;
+
+	unicode_char *context_buf;
+	size_t context_buf_len;
+
+	size_t context_buf_head;
+	size_t context_buf_tail;
+
+	struct searchresults_match_context *matched_context_head;
+	struct searchresults_match_context **matched_context_tail;
+
+	struct maildir_searchengine *se;
+
+};
+
+static MATCHEDSTR *creatematches(struct searchresults *sr);
+
+static int searchresults_init(struct searchresults *sr,
+			      struct maildir_searchengine *se);
+
+static void searchresults_destroy(struct searchresults *sr);
+
+static int do_maildir_search(const char *filename,
+			     struct searchresults *sr);
+
+
+static void execute_maildir_search(const char *dirname,
+				   size_t pos,
+				   const char *searchtxt,
+				   const char *charset,
+				   unsigned nfiles,
+				   MSGINFO ***retval,
+				   MATCHEDSTR ***retcontext,
+				   unsigned long *last_message_searched)
+{
+	MSGINFO	**msginfo;
+	MATCHEDSTR **matches;
+	unsigned long i;
+	unsigned j;
+	char *utf8str;
+	char *p, *q;
+
+	if ((*retval=msginfo=malloc(sizeof(MSGINFO *)*nfiles)) == 0)
+		enomem();
+
+	if ((*retcontext=matches=malloc(sizeof(MATCHEDSTR *)*nfiles)) == 0)
+	{
+		free(msginfo);
+		enomem();
+	}
+
+	for (i=0; i<nfiles; i++)
+	{
+		msginfo[i]=0;
+		matches[i]=0;
+	}
+
+	if (opencache(dirname, "W"))	return;
+
+	if (pos >= all_cnt) return;
+
+	utf8str=libmail_u_convert_toutf8(searchtxt, charset, NULL);
+
+	if (!utf8str)
+		return;
+
+	/* Normalize whitespace in the search string */
+
+	p=q=utf8str;
+
+	while (*p && strchr(spaces, *p))
+		++p;
+
+	while (*p)
+	{
+		while (*p && !strchr(spaces, *p))
+		{
+			*q++ = *p++;
+		}
+
+		while (*p && strchr(spaces, *p))
+			++p;
+
+		if (*p)
+			*q++=' ';
+	}
+	*q=0;
+
+	if (*utf8str)
+	{
+		struct maildir_searchengine se;
+		int rc=-1;
+		unicode_char *ustr;
+		size_t ustr_size;
+		libmail_u_convert_handle_t h;
+
+		maildir_search_init(&se);
+
+		h=libmail_u_convert_tou_init("utf-8",
+					     &ustr,
+					     &ustr_size,
+					     1);
+
+		if (h)
+		{
+			libmail_u_convert(h, utf8str, strlen(utf8str));
+			if (libmail_u_convert_deinit(h, NULL) == 0)
+			{
+				size_t n;
+
+				for (n=0; ustr[n]; ++n)
+					ustr[n]=unicode_lc(ustr[n]);
+
+				rc=maildir_search_start_unicode(&se, ustr);
+				free(ustr);
+			}
+		}
+
+		j=0;
+
+		for (i=0, j=0; rc == 0 && pos+i<all_cnt && j<nfiles; ++i)
+		{
+			MSGINFO *info=get_msginfo_alloc(pos+i);
+			char *filename;
+
+			if (!info)
+				continue;
+
+			*last_message_searched= pos+i;
+
+			filename=maildir_find(dirname, info->filename);
+
+			maildir_search_reset(&se);
+
+			if (filename)
+			{
+				struct searchresults sr;
+
+				info->msgnum=pos+i;
+
+				if (searchresults_init(&sr, &se) == 0)
+				{
+					if (do_maildir_search(filename, &sr))
+					{
+						msginfo[j]=info;
+						matches[j]=creatematches(&sr);
+						info=NULL;
+						++j;
+					}
+
+					searchresults_destroy(&sr);
+				}
+				free(filename);
+			}
+
+			if (info)
+				maildir_nfreeinfo(info);
+		}
+		maildir_search_destroy(&se);
+	}
+
+	free(utf8str);
+}
+
+#define sr_context_buf_index_inc(sr,n) ( ( (n)+1 ) % ( (sr)->context_buf_len))
+#define sr_context_buf_index_dec(sr,n) ( ( (n)+ (sr)->context_buf_len - 1 ) % ( (sr)->context_buf_len))
+
+static int searchresults_init(struct searchresults *sr,
+			      struct maildir_searchengine *se)
+{
+	sr->prevchar=0;
+	sr->foundcnt=0;
+	sr->finished=0;
+	sr->se=se;
+	sr->utf8buf_cnt=0;
+
+	sr->context_buf_len=maildir_search_len(se)+SEARCH_MATCH_CONTEXT_LEN*2+1;
+	sr->context_buf=malloc(sr->context_buf_len * sizeof(unicode_char));
+
+	if (sr->context_buf == NULL)
+		return -1;
+	sr->context_buf_head=0;
+	sr->context_buf_tail=0;
+
+	sr->matched_context_head=NULL;
+	sr->matched_context_tail=&sr->matched_context_head;
+
+	return 0;
+}
+
+static void searchresults_destroy(struct searchresults *sr)
+{
+	while (sr->matched_context_head)
+	{
+		struct searchresults_match_context *c=sr->matched_context_head;
+
+		sr->matched_context_head=c->next;
+
+		free(c->match_context_before);
+		free(c->match_context_after);
+		free(c->match_context);
+	}
+
+	free(sr->context_buf);
+}
+
+/* Save context before, and the matched context */
+
+static void search_found_save_context(struct searchresults *sr)
+{
+	struct searchresults_match_context *c=
+		malloc(sizeof(struct searchresults_match_context));
+	size_t n, i, j;;
+
+	if (c == NULL)
+		return;
+
+	if ((c->match_context_before=malloc((SEARCH_MATCH_CONTEXT_LEN+1)
+					    * sizeof(unicode_char))) == NULL)
+	{
+		free(c);
+		return;
+	}
+
+	if ((c->match_context=malloc((maildir_search_len(sr->se)+1)
+				     * sizeof(unicode_char))) == NULL)
+	{
+		free(c->match_context_before);
+		free(c);
+		return;
+	}
+
+	if ((c->match_context_after=malloc((SEARCH_MATCH_CONTEXT_LEN+1)
+					   * sizeof(unicode_char))) == NULL)
+	{
+		free(c->match_context);
+		free(c->match_context_before);
+		free(c);
+		return;
+	}
+
+	c->next=NULL;
+
+	*sr->matched_context_tail=c;
+	sr->matched_context_tail= &c->next;
+
+	/*
+	** Subtract from the head of the context buffer to arrive at the
+	** start of the matched context
+	*/
+
+	n=sr->context_buf_head;
+
+	for (i=maildir_search_len(sr->se); i > 0; --i)
+	{
+		if (n == sr->context_buf_tail)
+			break; /* Shouldn't happen */
+
+		n=sr_context_buf_index_dec(sr, n);
+	}
+
+	/* From here to the head is the matched context */
+
+	j=0;
+	for (i=n; i != sr->context_buf_head; )
+	{
+		c->match_context[j++]=sr->context_buf[i];
+		i=sr_context_buf_index_inc(sr, i);
+	}
+	c->match_context[j]=0;
+
+	/* Now, look before the start of the matched context */
+
+	for (i=n, j=0; j<SEARCH_MATCH_CONTEXT_LEN; ++j)
+	{
+		if (i == sr->context_buf_tail)
+			break; /* Possible */
+
+		i=sr_context_buf_index_dec(sr, i);
+	}
+
+	j=0;
+	while (i != n)
+	{
+		c->match_context_before[j++]=sr->context_buf[i];
+		i=sr_context_buf_index_inc(sr, i);
+	}
+	c->match_context_before[j]=0;
+
+	c->match_context_after_len=0;
+	c->match_context_after_max_len=SEARCH_MATCH_CONTEXT_LEN;
+}
+
+static int do_search_utf8(struct searchresults *res)
+{
+	unicode_char *uc=NULL;
+	size_t n;
+	libmail_u_convert_handle_t h;
+
+	if (res->utf8buf_cnt == 0)
+		return 0;
+
+	if (res->finished)
+		return -1;
+
+	res->utf8buf[res->utf8buf_cnt]=0;
+	res->utf8buf_cnt=0;
+
+	h=libmail_u_convert_tou_init("utf-8",
+				     &uc,
+				     &n,
+				     1);
+
+	if (h)
+	{
+		libmail_u_convert(h, res->utf8buf, strlen(res->utf8buf));
+		if (libmail_u_convert_deinit(h, NULL) == 0)
+			;
+		else
+			uc=NULL;
+	}
+
+	for (n=0; uc && uc[n]; n++)
+	{
+		struct searchresults_match_context *c;
+
+		unicode_char origch=uc[n];
+		unicode_char ch=unicode_lc(origch);
+
+		maildir_search_step_unicode(res->se, ch);
+
+		/* Record the context, one char at a time */
+
+		res->context_buf[res->context_buf_head]=origch;
+		res->context_buf_head =
+			sr_context_buf_index_inc(res, res->context_buf_head);
+
+		if (res->context_buf_head == res->context_buf_tail)
+			res->context_buf_tail =
+				sr_context_buf_index_inc(res, res->context_buf_tail);
+		/* Accumulate post-match context for matched hits */
+
+		for (c=res->matched_context_head; c; c=c->next)
+		{
+			if (c->match_context_after_len <
+			    c->match_context_after_max_len)
+				c->match_context_after[c->match_context_after_len++]=origch;
+		}
+
+		if (maildir_search_found(res->se))
+		{
+			search_found_save_context(res);
+			if (++res->foundcnt > 3)
+			{
+				res->finished=1;
+				break;
+			}
+
+			maildir_search_reset(res->se);
+		}
+	}
+
+	if (uc)
+		free(uc);
+
+	if (res->finished)
+		return 1;
+
+	return 0;		
+}
+
+static int do_search(const char *str, size_t n, void *arg)
+{
+	struct searchresults *res=(struct searchresults *)arg;
+
+	int rc=0;
+
+	while (n && rc == 0)
+	{
+		char ch=*str++;
+		--n;
+
+		if (strchr(spaces, ch))
+		{
+			ch=' ';
+
+			if (res->prevchar == ' ')
+				continue;
+		}
+		res->prevchar=ch;
+
+		if (res->utf8buf_cnt >= sizeof(res->utf8buf)-1)
+		{
+			size_t n;
+			char save_ch;
+			size_t save_n;
+
+			for (n=res->utf8buf_cnt-1;
+			     n > sizeof(res->utf8buf_cnt)/2; --n)
+				if ((unsigned char)((res->utf8buf[n] ^ 0xC0)
+						    & 0xC0))
+					break;
+
+			save_n=res->utf8buf_cnt;
+			save_ch=res->utf8buf[n];
+
+			res->utf8buf_cnt=n;
+
+			rc=do_search_utf8(res);
+
+			if (rc)
+				return rc;
+
+			res->utf8buf[n]=save_ch;
+
+			while (n < save_n)
+			{
+				res->utf8buf[res->utf8buf_cnt++]=
+					res->utf8buf[n];
+				++n;
+			}
+		}
+
+		res->utf8buf[res->utf8buf_cnt]=ch;
+		++res->utf8buf_cnt;
+	}
+
+	return rc;
+}
+
+static int do_maildir_search(const char *filename,
+			     struct searchresults *sr)
+{
+	struct rfc2045src *src;
+	struct rfc2045_decodemsgtoutf8_cb cb;
+	int fd=maildir_semisafeopen(filename, O_RDONLY, 0);
+	struct rfc2045 *rfc2045p;
+
+	if (fd < 0)
+		return 0;
+
+	rfc2045p=rfc2045_fromfd(fd);
+
+	if (rfc2045p == NULL)
+	{
+		close(fd);
+		return 0;
+	}
+
+	memset(&cb, 0, sizeof(cb));
+	cb.output_func=do_search;
+	cb.arg=sr;
+
+	if ((src=rfc2045src_init_fd(fd)) != NULL)
+	{
+		if (rfc2045_decodemsgtoutf8(src, rfc2045p, &cb) == 0)
+			do_search_utf8(sr);
+		rfc2045src_deinit(src);
+	}
+
+	rfc2045_free(rfc2045p);
+	close(fd);
+	return sr->foundcnt ? 1:0;
+}
+
+static void savematch(struct searchresults_match_context *smc,
+		      MATCHEDSTR **curptr);
+
+static MATCHEDSTR *creatematches(struct searchresults *sr)
+{
+	size_t n=0;
+	struct searchresults_match_context *smc;
+	MATCHEDSTR *retval, *retptr;
+
+	/* Count, allocate the array */
+	for (smc=sr->matched_context_head; smc; smc=smc->next)
+		++n;
+
+	if ((retval=malloc(sizeof(MATCHEDSTR)*(n+1))) == NULL)
+		return NULL;
+
+	retptr=retval;
+
+	for (smc=sr->matched_context_head; smc; smc=smc->next)
+	{
+		savematch(smc, &retptr);
+	}
+
+	/* Last one */
+
+	retptr->prefix=NULL;
+	retptr->match=NULL;
+	retptr->suffix=NULL;
+	return retval;
+}
+
+static char *match_conv(const unicode_char *uc)
+{
+	char *cbuf;
+	size_t csize;
+	libmail_u_convert_handle_t h;
+	size_t i;
+
+	if ((h=libmail_u_convert_fromu_init("utf-8", &cbuf, &csize, 1)) == NULL)
+		return NULL;
+
+	for (i=0; uc[i]; ++i)
+		;
+
+	libmail_u_convert_uc(h, uc, i);
+
+	if (libmail_u_convert_deinit(h, NULL))
+		return NULL;
+	return cbuf;
+}
+
+static void savematch(struct searchresults_match_context *smc,
+		      MATCHEDSTR **curptr)
+{
+	if ( ((*curptr)->prefix=match_conv(smc->match_context_before))
+	     == NULL)
+		return;
+
+	if ( ((*curptr)->match=match_conv(smc->match_context)) == NULL)
+	{
+		free( (*curptr)->prefix );
+		return;
+	}
+
+	smc->match_context_after[smc->match_context_after_len]=0;
+
+	if ( ((*curptr)->suffix=match_conv(smc->match_context_after))
+	     == NULL)
+	{
+		free( (*curptr)->match );
+		free( (*curptr)->prefix );
+		return;
+	}
+	++ (*curptr);
+}
+
 static void dodirscan(const char *, const char *, unsigned *, unsigned *);
 
 void maildir_count(const char *folder,
@@ -2062,8 +2965,7 @@ int	fd;
 		{
 			char *uibuf=rfc822_display_hdrvalue_tobuf("subject",
 								  val,
-								  unicode_UTF8
-								  .chset,
+								  "utf-8",
 								  NULL, NULL);
 
 			if (mi->subject_s)	free(mi->subject_s);
@@ -2113,7 +3015,7 @@ int	fd;
 				}
 
 				p=rfc822_display_name_tobuf(from_addra, cnt,
-							    unicode_UTF8.chset);
+							    "utf-8");
 			}
 
 			if (p)
@@ -2981,6 +3883,27 @@ void maildir_cleanup()
 	closedb();
 }
 
+void matches_free(MATCHEDSTR **p, unsigned n)
+{
+	size_t i;
+
+	for (i=0; p && i<n; ++i)
+	{
+		MATCHEDSTR *q;
+
+		for (q=p[i]; q && q->match; ++q)
+		{
+			free(q->prefix);
+			free(q->match);
+			free(q->suffix);
+		}
+		if (p[i])
+			free(p[i]);
+	}
+	if (p)
+		free(p);
+}
+
 /*
 ** Convert folder names to modified-UTF7 encoding.
 */
@@ -2988,23 +3911,19 @@ void maildir_cleanup()
 char *folder_toutf7(const char *foldername)
 {
 	char *p;
+	int converr;
 
-#if UTF7_FOLDER_ENCODING
-	unicode_char *uc;
-	int errflag;
-	const struct unicode_info *uiptr;
+	p=libmail_u_convert_tobuf(foldername, sqwebmail_content_charset,
+				  unicode_x_imap_modutf7, &converr);
 
-	if ((uiptr=unicode_find(sqwebmail_content_charset)) != 0 &&
-	    (uc=(*uiptr->c2u)(uiptr, foldername, &errflag)) != 0)
+	if (p && converr)
 	{
-		p=unicode_uctomodutf7(uc);
-
-		free(uc);
-		if (!p)
-			enomem();
-		return (p);
+		free(p);
+		p=NULL;
 	}
-#endif
+
+	if (p)
+		return (p);
 
 	p=strdup(foldername);
 	if (!p)
@@ -3015,21 +3934,21 @@ char *folder_toutf7(const char *foldername)
 char *folder_fromutf7(const char *foldername)
 {
 	char *p;
+	int converr;
 
-#if UTF7_FOLDER_ENCODING
-	unicode_char *uc;
-	int errflag;
-	const struct unicode_info *uiptr;
+	p=libmail_u_convert_tobuf(foldername,
+				  unicode_x_imap_modutf7,
+				  sqwebmail_content_charset,
+				  &converr);
 
-	if ((uiptr=unicode_find(sqwebmail_content_charset)) != 0 &&
-	    (uc=unicode_modutf7touc(foldername, &errflag)) != 0)
+	if (p && converr)
 	{
-		p=(*uiptr->u2c)(uiptr, uc, NULL);
-		free(uc);
-		if (p)
-			return (p);
+		free(p);
+		p=NULL;
 	}
-#endif
+
+	if (p)
+		return (p);
 
 	p=strdup(foldername);
 	if (!p)
