@@ -1,9 +1,9 @@
+#include "config.h"
 /*
-** $Id: mailfilter.c,v 1.20 2006/09/09 01:24:09 mrsam Exp $
 */
 
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2010 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -48,6 +48,7 @@ static void clrfields()
 	cgi_put("autoresponse_dupe", "");
 	cgi_put("autoresponse_days", "");
 	cgi_put("autoresponse_from", "");
+	cgi_put("autoresponse_noquote", "");
 }
 
 void mailfilter_list()
@@ -115,6 +116,7 @@ struct maildirfilterrule *r;
 	if (*cgi("do.add"))
 		clrfields();
 	memset(&mf, 0, sizeof(mf));
+
 	if (maildir_filter_loadmaildirfilter(&mf, "."))
 	{
 		maildir_filter_freerules(&mf);
@@ -311,6 +313,9 @@ struct maildirfilterrule *r;
 			if (!fromhdr)
 				enomem();
 			cgi_put("autoresponse_from", fromhdr);
+
+			if (mfai.noquote)
+				cgi_put("autoresponse_noquote", "1");
 		}
 		else if (strcmp(actionbuf, "exit") == 0)
 		{
@@ -529,6 +534,11 @@ const char *autoreply_from="";
 			mfaii.days=atoi(p);
 		}
 
+		p=cgi("autoresponse_noquote");
+
+		if (*p)
+			mfaii.noquote=1;
+
 		q=maildir_filter_autoresp_info_asstr(&mfaii);
 		maildir_filter_autoresp_info_free(&mfaii);
 
