@@ -3032,7 +3032,8 @@ static void disposition_attachment(FILE *fp, const char *p, int attachment)
 }
 
 
-void msg2html_download(FILE *fp, const char *mimeid, int dodownload)
+void msg2html_download(FILE *fp, const char *mimeid, int dodownload,
+		       const char *system_charset)
 {
 	struct	rfc2045 *rfc, *part;
 	char	buf[BUFSIZ];
@@ -3057,7 +3058,7 @@ void msg2html_download(FILE *fp, const char *mimeid, int dodownload)
 
 	rfc2045_mimeinfo(part, &content_type, &dummy, &charset);
 
-	if (rfc2231_udecodeType(part, "name", NULL,
+	if (rfc2231_udecodeType(part, "name", system_charset,
 				&content_name) < 0)
 		content_name=NULL;
 
@@ -3066,13 +3067,15 @@ void msg2html_download(FILE *fp, const char *mimeid, int dodownload)
 		char *disposition_filename;
 		const char *p;
 
-		if (rfc2231_udecodeDisposition(part, "filename", NULL,
+		if (rfc2231_udecodeDisposition(part, "filename",
+					       (strncmp(content_type, "text/",
+							5) == 0 ?
+						charset:system_charset),
 					       &disposition_filename) < 0)
 		{
 			if (content_name)
 				free(content_name);
-			rfc2045_free(rfc);
-			return;
+			disposition_filename=NULL;
 		}
 
 
@@ -3082,7 +3085,8 @@ void msg2html_download(FILE *fp, const char *mimeid, int dodownload)
 		if (!p || !*p) p="message.dat";
 		disposition_attachment(stdout, p, 1);
 		content_type="application/octet-stream";
-		free(disposition_filename);
+		if (disposition_filename)
+			free(disposition_filename);
 	} else {
 		if (content_name && *content_name)
 			disposition_attachment(stdout, content_name, 0);
@@ -3092,7 +3096,9 @@ void msg2html_download(FILE *fp, const char *mimeid, int dodownload)
 		content_name && *content_name ?
 		"Content-Type: %s; charset=%s; name=\"%s\"\n\n":
 		"Content-Type: %s; charset=%s\n\n",
-			content_type, charset, content_name ? content_name:"");
+		content_type,
+		charset,
+		content_name ? content_name:"");
 	if (content_name)
 		free(content_name);
 
