@@ -1,11 +1,10 @@
 /*
-** Copyright 1998 - 2009 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: newmsg.c,v 1.69 2009/11/14 21:15:43 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -62,7 +61,7 @@ extern void output_urlencoded(const char *);
 extern char *newmsg_newdraft(const char *, const char *, const char *,
 				const char *);
 extern char *newmsg_createdraft(const char *);
-extern char *newmsg_createsentmsg(const char *, int *, int);
+extern char *newmsg_createsentmsg(const char *, int *);
 extern int ishttps();
 
 static void newmsg_header(const char *label, const char *field,
@@ -71,8 +70,6 @@ static void newmsg_header(const char *label, const char *field,
 int		hdrmaxlen=512;
 const char	*p=getarg("HDRMAXLEN");
 
-const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
-
 	if (p && (atoi(p) > hdrmaxlen))
 		hdrmaxlen=atoi(p);
 
@@ -90,7 +87,7 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 
 		s=rfc822_display_hdrvalue_tobuf("subject",
 						encoded,
-						uiptr->chset,
+						sqwebmail_content_charset,
 						NULL,
 						NULL);
 
@@ -185,45 +182,176 @@ void newmsg_hiddenheader(const char *label, const char *value)
 
 /* ---------------------------------------------------- */
 
-/*
-** Strip leading space from flowed text format-encoded message.
-*/
+static size_t show_textarea_start_of_line(struct show_textarea_info *,
+					  const char *, size_t);
 
-static int textarea_eol;
+static size_t show_textarea_quoted_text(struct show_textarea_info *,
+					const char *, size_t);
 
-static int show_textarea(const char *p, size_t l, void *voidptr)
+static size_t show_textarea_notseen_sp(struct show_textarea_info *,
+				       const char *, size_t);
+
+static size_t show_textarea_seen_sp(struct show_textarea_info *,
+				    const char *, size_t);
+
+static size_t show_textarea_seen_spnl(struct show_textarea_info *,
+				      const char *, size_t);
+
+static size_t show_textarea_check_sig(struct show_textarea_info *,
+				      const char *, size_t);
+
+static size_t show_textarea_ignore_sig(struct show_textarea_info *,
+				       const char *, size_t);
+
+void show_textarea_init(struct show_textarea_info *info,
+			int stop_at_sig)
 {
-	const char *ptr=p;
-	size_t cnt=0;
+	info->handler=show_textarea_start_of_line;
+	info->stop_at_sig=stop_at_sig;
+}
 
-	while (l)
+void show_textarea(struct show_textarea_info *info,
+		   const char *ptr, size_t cnt)
+{
+	while (cnt)
 	{
-		if (textarea_eol)
-		{
-			textarea_eol=0;
-			if (*p == ' ')
-			{
-				if (cnt)
-					print_attrencodedlen(ptr, cnt,
-							     1, stdout);
-				++p;
-				--l;
-				ptr=p;
-				cnt=0;
-				continue;
-			}
-		}
+		size_t n= (*info->handler)(info, ptr, cnt);
 
-		if (*p == '\n')
-			textarea_eol=1;
-		++p;
-		--l;
-		++cnt;
+		cnt -= n;
+		ptr += n;
+	}
+}
+
+static size_t show_textarea_start_of_line(struct show_textarea_info *info,
+					  const char *ptr, size_t cnt)
+{
+	if (*ptr == '>')
+	{
+		info->handler=show_textarea_quoted_text;
+		return show_textarea_quoted_text(info, ptr, cnt);
 	}
 
-	if (cnt)
-		print_attrencodedlen(ptr, cnt, 1, stdout);
-	return (0);
+	info->handler=show_textarea_notseen_sp;
+
+	if (*ptr == ' ')
+		return 1; /* Consume space-stuffed space */
+
+	if (info->stop_at_sig)
+	{
+		info->handler=show_textarea_check_sig;
+		info->sig_index=0;
+		return show_textarea_check_sig(info, ptr, cnt);
+	}
+	return show_textarea_notseen_sp(info, ptr, cnt);
+}
+
+static size_t show_textarea_quoted_text(struct show_textarea_info *info,
+					const char *ptr, size_t cnt)
+{
+	size_t i;
+
+	for (i=0; i<cnt; i++)
+	{
+		if (ptr[i] == '\n')
+		{
+			++i;
+			info->handler=show_textarea_start_of_line;
+			break;
+		}
+	}
+
+	if (i)
+		print_attrencodedlen(ptr, i, 1, stdout);
+	return i;
+}
+
+static size_t show_textarea_notseen_sp(struct show_textarea_info *info,
+				       const char *ptr, size_t cnt)
+{
+	size_t i;
+
+	for (i=0; i<cnt; i++)
+	{
+		if (ptr[i] == '\n')
+		{
+			++i;
+			info->handler=show_textarea_start_of_line;
+			break;
+		}
+
+		if (ptr[i] == ' ')
+		{
+			if (i)
+				print_attrencodedlen(ptr, i, 1, stdout);
+			info->handler=show_textarea_seen_sp;
+			return i+1;
+		}
+	}
+
+	if (i)
+		print_attrencodedlen(ptr, i, 1, stdout);
+	return i;
+}
+
+static size_t show_textarea_seen_sp(struct show_textarea_info *info,
+				    const char *ptr, size_t cnt)
+{
+	if (*ptr != '\n')
+	{
+		info->handler=show_textarea_notseen_sp;
+		print_attrencodedlen(" ", 1, 1, stdout);
+		return show_textarea_notseen_sp(info, ptr, cnt);
+	}
+
+	info->handler=show_textarea_seen_spnl;
+	return 1;
+}
+
+static size_t show_textarea_seen_spnl(struct show_textarea_info *info,
+				      const char *ptr, size_t cnt)
+{
+	if (*ptr == '>' || *ptr == '\n') /* Fix this */
+	{
+		print_attrencodedlen("\n", 1, 1, stdout);
+		return show_textarea_start_of_line(info, ptr, cnt);
+	}
+
+	info->handler=show_textarea_notseen_sp;
+	return show_textarea_notseen_sp(info, ptr, cnt);
+}
+
+static size_t show_textarea_check_sig(struct show_textarea_info *info,
+				      const char *ptr, size_t cnt)
+{
+	static const char sig[]={'-', '-', ' ', '\n'};
+	size_t i;
+
+	for (i=0; i<cnt; ++i)
+	{
+		if (i + info->sig_index >= sizeof(sig))
+		{
+			info->handler=show_textarea_ignore_sig;
+			return cnt;
+		}
+
+		if (ptr[i] != sig[i+info->sig_index])
+		{
+			info->handler=show_textarea_notseen_sp;
+
+			show_textarea(info, sig, info->sig_index);
+			show_textarea(info, ptr, cnt);
+			return cnt;
+		}
+	}
+
+	info->sig_index += cnt;
+	return cnt;
+}
+
+static size_t show_textarea_ignore_sig(struct show_textarea_info *info,
+				       const char *ptr, size_t cnt)
+{
+	return cnt;
 }
 
 /*
@@ -317,6 +445,12 @@ char *newmsg_alladdrs(FILE *fp)
 	return (p);
 }
 
+static int show_textarea_trampoline(const char *ptr, size_t cnt, void *arg)
+{
+	show_textarea( (struct show_textarea_info *)arg, ptr, cnt);
+	return 0;
+}
+
 void newmsg_showfp(FILE *fp, int *attachcnt)
 {
 	struct	rfc2045 *p=rfc2045_fromfp(fp), *q;
@@ -347,11 +481,25 @@ void newmsg_showfp(FILE *fp, int *attachcnt)
 
 	q=rfc2045_searchcontenttype(p, "text/plain");
 
-	textarea_eol=1;
 	if (q)
-		rfc2045_decodetextmimesection(fileno(fp), q,
-					      sqwebmail_content_charset,
-					      &show_textarea, NULL);
+	{
+		struct rfc2045src *src=rfc2045src_init_fd(fileno(fp));
+
+		if (src)
+		{
+			struct show_textarea_info info;
+
+			show_textarea_init(&info, 1);
+
+			rfc2045_decodetextmimesection(src, q,
+						      sqwebmail_content_charset,
+						      NULL,
+						      &show_textarea_trampoline,
+						      &info);
+			rfc2045src_deinit(src);
+			show_textarea(&info, "\n", 1);
+		}
+	}
 	rfc2045_free(p);
 }
 
@@ -641,26 +789,6 @@ void newmsg_init(const char *folder, const char *pos)
 	else
 	{
 		printf("%s", cgi("body"));
-		if ((fp=fopen(SIGNATURE, "r")) != NULL)
-		{
-		char	buf[256];
-		int	n;
-		struct stat stat_buf;
-		unsigned long filesize=0;
-
-			if (stat(SIGNATURE, &stat_buf) == 0)
-				filesize=stat_buf.st_size;
-			if (filesize > 0)
-				printf("\n\n");
-
-			while ((n=fread(buf, 1, sizeof(buf)-1, fp)) > 0)
-			{
-				buf[n]=0;
-				output_attrencoded_oknl(buf);
-			}
-			fclose(fp);
-			fp=NULL;
-		}
 	}
 	printf("%s\n", text2);
 
@@ -812,7 +940,7 @@ int dsn;
 	if (!draftmessage)
 		enomem();
 
-	filename=newmsg_createsentmsg(draftmessage, &isgpgerr, 1);
+	filename=newmsg_createsentmsg(draftmessage, &isgpgerr);
 
 	if (!filename)
 	{
