@@ -1,11 +1,10 @@
 /*
-** Copyright 1998 - 2004 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: newmsg_newdraft.c,v 1.41 2009/11/14 21:15:43 mrsam Exp $
 */
 #include	"config.h"
 #include	"cgi/cgi.h"
@@ -45,20 +44,6 @@ static void writefunc(const char *p, size_t l, void *dummy)
 	maildir_writemsg(draftfd, p, l);
 }
 
-static void newmsg_writesig(void *dummy)
-{
-FILE	*fp=fopen(SIGNATURE, "r");
-char	buf[256];
-int	n;
-
-	if (!fp)	return;
-	maildir_writemsg(draftfd, "\n\n", 2);
-	while ((n=fread(buf, 1, sizeof(buf), fp)) > 0)
-		maildir_writemsg(draftfd, buf, n);
-	fclose(fp);
-	maildir_writemsg(draftfd, "\n", 1);
-}
-
 char *newmsg_newdraft(const char *folder, const char *pos,
 			const char *forwardsep, const char *replysalut)
 {
@@ -156,24 +141,28 @@ int	x;
 	{
 		char *ml=getmailinglists();
 		struct rfc2045_mkreplyinfo ri;
+		struct rfc2045src *src;
 		int rc;
 
+		src=rfc2045src_init_fd(fileno(fp));
+		if (src == NULL)
+			enomem();
+
 		memset(&ri, 0, sizeof(ri));
-		ri.fd=fileno(fp);
+		ri.src=src;
 		ri.rfc2045partp=rfc2045partp;
 		ri.replymode=replymode;
 		ri.replysalut=replysalut;
 		ri.forwardsep=forwardsep;
 		ri.myaddr_func=ismyaddr;
 		ri.write_func=writefunc;
-		ri.writesig_func=newmsg_writesig;
 		ri.mailinglists=ml;
 		ri.charset=sqwebmail_content_charset;
 
 		if (strcmp(replymode, "forward") == 0
 		    || strcmp(replymode, "forwardatt") == 0)
 		{
-			rc=rfc2045_makereply_unicode(&ri);
+			rc=rfc2045_makereply(&ri);
 		}
 		else
 		{
@@ -187,9 +176,10 @@ int	x;
 			free(basename);
 			maildir_writemsgstr(draftfd, "\n");
 
-			rc=rfc2045_makereply_unicode(&ri);
+			rc=rfc2045_makereply(&ri);
 		}
 		free(ml);
+		rfc2045src_deinit(src);
 
 		if (rc)
 		{
