@@ -5,7 +5,6 @@
 
 
 /*
-** $Id: pcp.c,v 1.32 2010/03/19 01:09:26 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"pcp.h"
@@ -72,7 +71,7 @@ extern void output_form(const char *);
 extern void newmsg_preview(const char *);
 extern void output_urlencoded(const char *);
 extern void attachments_head(const char *, const char *, const char *);
-extern char *newmsg_createsentmsg(const char *, int *, int);
+extern char *newmsg_createsentmsg(const char *, int *);
 extern const char *sqwebmail_mailboxid;
 extern char *scriptptrget();
 extern void attach_delete(const char *);
@@ -2446,7 +2445,7 @@ void sqpcp_save()
 	}
 	fclose(fp);
 
-	sentmsg=newmsg_createsentmsg(msg, &isgpgerr, 0);
+	sentmsg=newmsg_createsentmsg(msg, &isgpgerr);
 
 	/* Immediately remove the formatted event text from the sent folder */
 
