@@ -5,7 +5,6 @@
 
 
 /*
-** $Id: pref.c,v 1.31 2008/02/03 15:25:06 mrsam Exp $
 */
 #include	"pref.h"
 #include	"config.h"
@@ -18,6 +17,8 @@
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
+#include	<sys/stat.h>
+#include	<unistd.h>
 
 #define	OLDEST1ST	"OLDEST1ST"
 #define	FULLHEADERS	"FULLHEADERS"
@@ -55,6 +56,7 @@ int pref_wikifmt;
 
 char *pref_from=0;
 char *pref_ldap=0;
+extern const char *sqwebmail_content_charset;
 
 #if ENABLE_WEBPASS
 extern int check_sqwebpass(const char *);
@@ -352,7 +354,16 @@ void pref_setprefs()
 		pref_init();
 		if ((fp=fopen(SIGNATURE, "w")) != NULL)
 		{
-			fprintf(fp, "%s", cgi("signature"));
+			char *sig_utf8=
+				libmail_u_convert_toutf8(cgi("signature"),
+							 sqwebmail_content_charset,
+							 NULL);
+
+			if (sig_utf8)
+			{
+				fprintf(fp, "%s", sig_utf8);
+				free(sig_utf8);
+			}
 			fclose(fp);
 		}
 
@@ -510,23 +521,60 @@ static const char selected[]=" selected='selected'";
 	printf("</select>\n");
 }
 
-void pref_signature()
+char *pref_getsig()
 {
-FILE	*fp;
-char	buf[256];
-int	n;
-
-	if ((fp=fopen(SIGNATURE, "r")) != NULL)
-	{
-		while ((n=fread(buf, 1, sizeof(buf)-1, fp)) > 0)
-		{
-			buf[n]=0;
-			output_attrencoded_oknl(buf);
-		}
-		fclose(fp);
-	}
+	return pref_getfile(fopen(SIGNATURE, "r"));
 }
 
+char *pref_getfile(FILE *fp)
+{
+	struct stat st;
+	char *utf8_buf;
+	char *sig_buf;
+
+	if (fp == NULL)
+		return NULL;
+
+	if (fstat(fileno(fp), &st) < 0)
+	{
+		fclose(fp);
+		return NULL;
+	}
+
+	utf8_buf=malloc(st.st_size+1);
+
+	if (!utf8_buf)
+	{
+		fclose(fp);
+		return NULL;
+	}
+
+	if (fread(utf8_buf, 1, st.st_size, fp) != st.st_size)
+	{
+		fclose(fp);
+		return NULL;
+	}
+	utf8_buf[st.st_size]=0;
+	fclose(fp);
+
+	sig_buf=libmail_u_convert_fromutf8(utf8_buf,
+					   sqwebmail_content_charset,
+					   NULL);
+	free(utf8_buf);
+
+	return sig_buf;
+}
+
+void pref_signature()
+{
+	char *p=pref_getsig();
+
+	if (p)
+	{
+		output_attrencoded_oknl(p);
+		free(p);
+	}
+}
 /*
 ** Get a setting from GPGCONFIGFILE
 **
