@@ -1,3 +1,4 @@
+#include "config.h"
 /*
 ** Copyright 1998 - 2001 Double Precision, Inc.  See COPYING for
 ** distribution information.
@@ -5,7 +6,6 @@
 
 
 /*
-** $Id: sqispell.c,v 1.23 2007/05/06 21:49:29 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"maildir.h"
@@ -22,6 +22,7 @@
 #include	<fcntl.h>
 #include	<ctype.h>
 
+extern const char *sqwebmail_content_charset;
 extern void output_form(const char *);
 extern const char *sqwebmail_content_ispelldict;
 extern void output_attrencoded(const char *);
@@ -385,6 +386,40 @@ static void showfunc(const char *p, size_t n, void *dummy)
 	}
 }
 
+static void show_part(const char *ptr, size_t cnt)
+{
+	unicode_char *uc;
+	size_t ucsize;
+	int conv_err;
+
+	if (libmail_u_convert_tou_tobuf(ptr, cnt,
+					sqwebmail_content_charset,
+					&uc,
+					&ucsize,
+					&conv_err) == 0)
+	{
+		if (conv_err)
+		{
+			free(uc);
+			uc=NULL;
+		}
+	}
+
+
+	if (uc)
+	{
+
+		struct filter_info info;
+
+		filter_start(&info, sqwebmail_content_charset,
+			     &showfunc, NULL);
+		filter(&info, uc, ucsize);
+		filter_end(&info);
+
+		free(uc);
+	}
+}
+
 void spell_show()
 {
 const char *draftmessage=cgi("draftmessage");
@@ -460,23 +495,10 @@ const char *finishlab=getarg("FINISH");
 		p=0;
 
 
-	{
-		struct filter_info info;
-
-		filter_start(&info, FILTER_FOR_DISPLAY, &showfunc, NULL);
-		filter(&info, ispellline+p, msp->word_pos-p);
-		filter_end(&info);
-	}
-
-	{
-		struct filter_info info;
-
-		printf("<strong>");
-		filter_start(&info, FILTER_FOR_DISPLAY, &showfunc, NULL);
-		filter(&info, ispellline+msp->word_pos, strlen(msp->misspelled_word));
-		filter_end(&info);
-		printf("</strong>");
-	}
+	show_part(ispellline+p, msp->word_pos-p);
+	printf("<strong>");
+	show_part(ispellline+msp->word_pos, strlen(msp->misspelled_word));
+	printf("</strong>");
 
 	p=msp->word_pos+strlen(msp->misspelled_word);
 	if (l-p < 30)
@@ -497,13 +519,7 @@ const char *finishlab=getarg("FINISH");
 		break;
 	}
 
-	{
-		struct filter_info info;
-
-		filter_start(&info, FILTER_FOR_DISPLAY, &showfunc, NULL);
-		filter(&info, ispellline+p, n);
-		filter_end(&info);
-	}
+	show_part(ispellline+p, n);
 
 	if (n != l-p)
 		printf("&nbsp;...");
