@@ -5,7 +5,6 @@
 
 
 /*
-** $Id: sqwebmail.c,v 1.147 2009/11/08 18:14:47 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"sqconfig.h"
@@ -1552,6 +1551,10 @@ static void do_output_form_loop(FILE *f)
 			listrights();
 		else if (strcmp(kw, "GETACL") == 0)
 			getacl();
+		else if (strcmp(kw, "MSGPOS") == 0)
+		{
+			printf("%ld", atol(cgi("pos"))+1);
+		}
 		else if (strncmp(kw, "radio:", 6) == 0)
 		{
 		const char *name=strtok(kw+6, ":");
@@ -1941,6 +1944,11 @@ char	*p;
 			formname="folders";	/* Naughty boy */
 	}
 
+	if (*cgi("do.search"))
+	{
+		folder_search(sqwebmail_folder, atol(cgi("pos")));
+		return;
+	}
 	p=malloc(strlen(formname)+6);
 	if (!p)	enomem();
 
@@ -2158,6 +2166,14 @@ int main(int argc, char **argv)
 	const char *prefork;
 	int n;
 
+#if 0
+	if (getenv("SQWEBMAIL_DEBUG"))
+	{
+		main2();
+		return (0);
+	}
+#endif
+
 	courier_authdebug_login_init();
 
 	if (argc > 1)
